@@ -2,6 +2,7 @@
 
 import shasta
 import GetConfig
+import ast
 import sys
 
 # Read the config file.
@@ -16,19 +17,10 @@ a.accessReadGraph()
 
 # Create vertices of the marker graph.
 a.createMarkerGraphVertices(
-    alignMethod = int(config['Align']['alignMethod']),
-    maxMarkerFrequency = int(config['Align']['maxMarkerFrequency']),
-    maxSkip = int(config['Align']['maxSkip']),
-    maxDrift = int(config['Align']['maxDrift']),
-    matchScore = int(config['Align']['matchScore']),
-    mismatchScore = int(config['Align']['mismatchScore']),
-    gapScore = int(config['Align']['gapScore']),
-    downsamplingFactor = float(config['Align']['downsamplingFactor']),
-    bandExtend = int(config['Align']['bandExtend']),
-    maxBand = int(config['Align']['maxBand']),
     minCoverage = int(config['MarkerGraph']['minCoverage']),
     maxCoverage = int(config['MarkerGraph']['maxCoverage']),
     minCoveragePerStrand = int(config['MarkerGraph']['minCoveragePerStrand']),
+    allowDuplicateMarkers = ast.literal_eval(config['MarkerGraph']['allowDuplicateMarkers']),
     peakFinderMinAreaFraction = float(config['MarkerGraph']['peakFinder.minAreaFraction']),
     peakFinderAreaStartIndex = int(config['MarkerGraph']['peakFinder.areaStartIndex']))
 a.findMarkerGraphReverseComplementVertices()
