@@ -12,7 +12,7 @@ a.accessMarkerGraphVertices()
 a.accessMarkerGraphReverseComplementVertex()
 a.accessMarkerGraphEdges(True, True)
 
-# ********************** Expose this as an expotion when code stabilizes
-a.createMarkerGraphSecondaryEdges(secondaryEdgeMaxSkip = 100)
+a.createMarkerGraphSecondaryEdges(
+    secondaryEdgeMaxSkip = int(config['MarkerGraph']['secondaryEdges.maxSkip']))
 
 
