@@ -6,9 +6,12 @@
 #include "AlignmentCandidates.hpp"
 #include "AssemblyGraph2Statistics.hpp"
 #include "HttpServer.hpp"
+#include "invalid.hpp"
 #include "Kmer.hpp"
+#include "MappedMemoryOwner.hpp"
 #include "Marker.hpp"
 #include "MarkerGraph.hpp"
+#include "MarkerGraphEdgePairInfo.hpp"
 #include "MemoryMappedObject.hpp"
 #include "MultithreadedObject.hpp"
 #include "ReadGraph.hpp"
@@ -25,7 +28,6 @@ namespace shasta {
 
     class Assembler;
     class AssemblerInfo;
-    class AssemblyGraph;
     class Alignment;
     class AlignmentData;
     class AlignmentGraph;
@@ -38,21 +40,28 @@ namespace shasta {
     class ConsensusCaller;
     class Histogram2;
     class InducedAlignment;
+    class KmerChecker;
+    class KmersOptions;
     class LocalAssemblyGraph;
     class LocalAlignmentCandidateGraph;
     class LocalAlignmentGraph;
-    class LocalMarkerGraph;
+    class LocalMarkerGraph0;
     class LocalReadGraph;
     class LocalReadGraphTriangles;
-    class LocalMarkerGraphRequestParameters;
+    class LocalMarkerGraph0RequestParameters;
     class LongBaseSequences;
     class MarkerConnectivityGraph;
     class MarkerConnectivityGraphVertexMap;
     class Mode2AssemblyOptions;
+    class Mode3AssemblyOptions;
+    class Mode3Assembler;
     class OrientedReadPair;
     class Reads;
     class ReferenceOverlapMap;
 
+    namespace mode0 {
+        class AssemblyGraph;
+    }
 
     namespace MemoryMapped {
         class ByteAllocator;
@@ -70,10 +79,6 @@ namespace shasta {
         class Options;
     }
 
-    namespace mode3 {
-        class AssemblyGraph;
-    }
-
     extern template class MultithreadedObject<Assembler>;
 }
 
@@ -102,6 +107,9 @@ public:
     // The length of k-mers used to define markers.
     size_t k;
 
+    // The method used to generate kmers (--Kmers.generationMethod).
+    uint64_t kmerGenerationMethod;
+
     // The page size in use for this run.
     size_t largeDataPageSize;
 
@@ -187,8 +195,9 @@ public:
 
 
 class shasta::Assembler :
-    public MultithreadedObject<Assembler>
-    , public HttpServer {
+    public MultithreadedObject<Assembler>,
+    public MappedMemoryOwner,
+    public HttpServer {
 public:
 
 
@@ -230,8 +239,6 @@ public:
     void findMarkers(size_t threadCount);
     void accessMarkers();
     void writeMarkers(ReadId, Strand, const string& fileName);
-    vector<KmerId> getMarkers(ReadId, Strand);
-    void writeMarkerFrequency();
 
     // Write the reads that overlap a given read.
     void writeOverlappingReads(ReadId, Strand, const string& fileName);
@@ -270,7 +277,6 @@ public:
     );
     void accessAlignmentData();
     void accessAlignmentDataReadWrite();
-    void writeAlignmentDetails() const;
 
 
     // Loop over all alignments in the read graph
@@ -322,9 +328,6 @@ public:
     vector< tuple<ReadId, Strand, uint32_t> >
         getGlobalMarkerGraphVertexMarkers(MarkerGraph::VertexId) const;
 
-    // Compute marker graph vertex coverage statistics by KmerId.
-    void vertexCoverageStatisticsByKmerId() const;
-
 
 
     // Approximate transitive reduction of the marker graph.
@@ -358,45 +361,6 @@ public:
 
 
 
-    // Approximate reverse transitive reduction of the marker graph.
-    // The goal is to remove local back-edges.
-    // This works similarly to transitive reduction,
-    // but in the opposite direction.
-    // This does the following:
-    // - Edges with coverage greater than lowCoverageThreshold
-    //   and less then highCoverageThreshold are processed in
-    //   ordered of increasing coverage:
-    //   * For each such edge A->B, we look for a path of length
-    //     at most maxDistance starting at B and ending at A
-    //     that does not use edge A->B and also does not use any
-    //     edges already marked wasRemovedByTransitiveReduction.
-    //   * If such a path is found, the edge is marked
-    //     wasRemovedByTransitiveReduction.
-    void reverseTransitiveReduction(
-        size_t lowCoverageThreshold,
-        size_t highCoverageThreshold,
-        size_t maxDistance);
-
-
-
-private:
-
-    // Data filled in by the constructor.
-    string largeDataFileNamePrefix;
-    size_t largeDataPageSize;
-
-    // Function to construct names for binary objects.
-    string largeDataName(const string& name) const
-    {
-        if(largeDataFileNamePrefix.empty()) {
-            return "";  // Anonymous;
-        } else {
-            return largeDataFileNamePrefix + name;
-        }
-    }
-
-
-
     // Various pieces of assembler information stored in shared memory.
     // See class AssemblerInfo for more information.
 public:
@@ -425,160 +389,44 @@ public:
 
     void computeReadIdsSortedByName();
 
+    // Find duplicate reads, as determined by name (not sequence).
+    // This also sets the isDuplicate and discardDueToDuplicates read flags
+    // and summarizes what it found Duplicates.csv.
+    void findDuplicateReads(const string& handleDuplicates);
+
 
 private:
 
 
 
-    // Table of all k-mers of length k.
-    // Among all 4^k k-mers of length k, we choose a subset
-    // that we call "markers".
-    // The value of k used is stored in assemblerInfo.
-    // The k-mer table is a vector of 4^k pairs,
-    // indexed by k-mer id as computed using Kmer::id(k).
-    // The markers are selected at the beginning of an assembly
-    // and never changed, and selected in such a way that,
-    // if (and only if) a k-mer is a marker, its reverse complement
-    // is also a marker. That is, for all permitted values of i, 0 <= i < 4^k:
-    // kmerTable[i].isMarker == kmerTable[kmerTable[i].reverseComplementKmerId].isMarker
-    MemoryMapped::Vector<KmerInfo> kmerTable;
-    void checkKmersAreOpen() const;
 
-public:
-    void accessKmers();
-    void writeKmers(const string& fileName) const;
-
-    // Select marker k-mers randomly.
-    void randomlySelectKmers(
-        size_t k,           // k-mer length.
-        double probability, // The probability that a k-mer is selected as a marker.
-        int seed            // For random number generator.
-    );
-
-
-
-    // Select marker k-mers randomly, but excluding
-    // the ones that have high frequency in the reads.
-    void selectKmersBasedOnFrequency(
-
-        // k-mer length.
-        size_t k,
-
-        // The desired marker density
-        double markerDensity,
-
-        // Seed for random number generator.
-        int seed,
-
-        // Exclude k-mers enriched by more than this amount.
-        // Enrichment is the ratio of k-mer frequency in reads
-        // over what a random distribution would give.
-        double enrichmentThreshold,
-
-        size_t threadCount
-    );
-
-
-
-    // In this version, marker k-mers are selected randomly, but excluding
-    // any k-mer that is over-enriched even in a single oriented read.
-    void selectKmers2(
-
-        // k-mer length.
-        size_t k,
-
-        // The desired marker density
-        double markerDensity,
-
-        // Seed for random number generator.
-        int seed,
-
-        // Exclude k-mers enriched by more than this amount,
-        // even in a single oriented read.
-        // Enrichment is the ratio of k-mer frequency in reads
-        // over what a random distribution would give.
-        double enrichmentThreshold,
-
-        size_t threadCount
-    );
-private:
-
-    class SelectKmers2Data {
+    // The KmerChecker can find out if a given KmerId is a marker.
+    shared_ptr<KmerChecker> kmerChecker;
     public:
+    void createKmerChecker(
+        const KmersOptions& kmersOptions,
+        uint64_t threadCount);
+    void accessKmerChecker();
 
-        double enrichmentThreshold;
+    // This one should eventually go away, but there are several scripts
+    // that depend on it.
+    void accessKmers()
+    {
+        accessKmerChecker();
+    }
 
-        // The number of times each k-mer appears in an oriented read.
-        // Indexed by KmerId.
-        MemoryMapped::Vector<uint64_t> globalFrequency;
-
-        // The number of oriented reads that each k-mer is
-        // over-enriched in by more than a factor enrichmentThreshold.
-        // Indexed by KmerId.
-        MemoryMapped::Vector<ReadId> overenrichedReadCount;
-
-    };
-    SelectKmers2Data selectKmers2Data;
-    void selectKmers2ThreadFunction(size_t threadId);
-
-
-
-    // In this version, marker k-mers are selected randomly, but excluding
-    // k-mers that appear repeated at short distances in any oriented read.
-    // More precisely, for each k-mer we compute the minimum distance
-    // (in RLE bases) at which any two copies of that k-mer appear in any oriented read.
-    // K-mers for which this minimum distance is less than distanceThreshold
-    // are not used as markers. Marker k-mers are selected randomly among the
-    // remaining k-mers, until the desired marker density is achieved.
-public:
-    void selectKmers4(
-
-        // k-mer length.
-        uint64_t k,
-
-        // The desired marker density
-        double markerDensity,
-
-        // Seed for random number generator.
-        uint64_t seed,
-
-        // Exclude k-mers that appear in any read in two copies,
-        // with the two copies closer than this distance (in RLE bases).
-        uint64_t distanceThreshold,
-
-        size_t threadCount
-    );
-private:
-    void selectKmers4ThreadFunction(size_t threadId);
-    class SelectKmers4Data {
-    public:
-
-        // The number of times each k-mer appears in an oriented read.
-        // Indexed by KmerId.
-        MemoryMapped::Vector<uint64_t> globalFrequency;
-
-        // The minimum distance at which two copies of each k-mer
-        // appear in any oriented read.
-        // Indexed by KmerId.
-        MemoryMapped::Vector< pair<std::mutex, uint32_t> > minimumDistance;
-
-    };
-    SelectKmers4Data selectKmers4Data;
-
-
-
-    // Read the k-mers from file.
-public:
-    void readKmersFromFile(uint64_t k, const string& fileName);
 
 private:
-    void computeKmerFrequency(size_t threadId);
-    void initializeKmerTable();
 
 
+    // Hash a KmerId in such a way that it has the same hash as its reverse
+    // complement. This is used by alignment method 3 to downsample markers.
+    uint32_t hashKmerId(KmerId) const;
 
     // The markers on all oriented reads. Indexed by OrientedReadId::getValue().
+public:
     MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t> markers;
+private:
     void checkMarkersAreOpen() const;
 
     // Get markers sorted by KmerId for a given OrientedReadId.
@@ -588,7 +436,9 @@ private:
 
     // Given a marker by its OrientedReadId and ordinal,
     // return the corresponding global marker id.
+public:
     MarkerId getMarkerId(OrientedReadId, uint32_t ordinal) const;
+private:
     MarkerId getReverseComplementMarkerId(OrientedReadId, uint32_t ordinal) const;
     MarkerId getMarkerId(const MarkerDescriptor& m) const
     {
@@ -606,20 +456,95 @@ private:
     // an additional 4 bytes per marker.
 public:
     pair<OrientedReadId, uint32_t> findMarkerId(MarkerId) const;
-private:
 
 
+    // KmerIds for all markers. Indexed by OrientedReadId::getValue().
+    // Only stored during alignment computation, and then freed.
+    MemoryMapped::VectorOfVectors<KmerId, uint64_t> markerKmerIds;
+    void computeMarkerKmerIds(uint64_t threadCount);
+    void cleanupMarkerKmerIds();
+private:
+    void computeMarkerKmerIdsThreadFunction(size_t threadId);
+
 
     // Pairs (KmerId, ordinal), sorted by KmerId, for each oriented read.
     // Indexed by orientedReadId.getValue().
     // Used by alignment method 4.
-    MemoryMapped::VectorOfVectors< pair<KmerId, uint32_t>, uint64_t> sortedMarkers;
 public:
+    MemoryMapped::VectorOfVectors< pair<KmerId, uint32_t>, uint64_t> sortedMarkers;
     void computeSortedMarkers(uint64_t threadCount);
     bool accessSortedMarkers();
 private:
-    void computeSortedMarkersThreadFunction1(size_t threadId);
-    void computeSortedMarkersThreadFunction2(size_t threadId);
+    void computeSortedMarkersThreadFunction(size_t threadId);
+    // void computeSortedMarkersThreadFunction1(size_t threadId);
+    // void computeSortedMarkersThreadFunction2(size_t threadId);
+
+
+
+    // Low frequency markers for each oriented read.
+    // This stores, for each oriented read, the ordinals corresponding
+    // to marker with low frequency (up to maxMarkerFrequency), sorted by KmerId.
+    // Used by alignment method 5. It is only stored durign alignment
+    // computation.
+public:
+    MemoryMapped::VectorOfVectors<uint32_t, uint64_t> lowFrequencyMarkers;
+    void computeLowFrequencyMarkers(uint64_t maxMarkerFrequency, uint64_t threadCount);
+    void computeLowFrequencyMarkers(
+        const span<const KmerId>&,  // The marker k-mers for the oriented reads (sorted by ordinal)
+        uint64_t maxMarkerFrequency,
+        vector<uint32_t>&);         // The ordinals of the low frequency markers, sorted by KmerId.
+private:
+    void computeLowFrequencyMarkersThreadFunctionPass1(uint64_t threadId);
+    void computeLowFrequencyMarkersThreadFunctionPass2(uint64_t threadId);
+    void computeLowFrequencyMarkersThreadFunctionPass12(uint64_t pass);
+    class ComputeLowFrequencyMarkersData {
+    public:
+        uint64_t maxMarkerFrequency;
+    };
+    ComputeLowFrequencyMarkersData computeLowFrequencyMarkersData;
+
+
+
+    // Low level functions to get marker Kmers/KmerIds of an oriented read.
+    // They are obtained from the reads and not from CompressedMarker::kmerId,
+    // which will soon go away.
+
+    // Get the marker Kmer for an oriented read and ordinal.
+    Kmer getOrientedReadMarkerKmer(OrientedReadId, uint32_t ordinal) const;
+    Kmer getOrientedReadMarkerKmerStrand0(ReadId, uint32_t ordinal) const;
+    Kmer getOrientedReadMarkerKmerStrand1(ReadId, uint32_t ordinal) const;
+
+    // Get the marker KmerId for an oriented read and ordinal.
+    KmerId getOrientedReadMarkerKmerId(OrientedReadId, uint32_t ordinal) const;
+
+    // Get all marker Kmers for an oriented read.
+    void getOrientedReadMarkerKmers(OrientedReadId, const span<Kmer>&) const;
+    void getOrientedReadMarkerKmersStrand0(ReadId, const span<Kmer>&) const;
+    void getOrientedReadMarkerKmersStrand1(ReadId, const span<Kmer>&) const;
+
+    // Get all marker KmerIds for an oriented read.
+    void getOrientedReadMarkerKmerIds(OrientedReadId, const span<KmerId>&) const;
+    void getOrientedReadMarkerKmerIdsStrand0(ReadId, const span<KmerId>&) const;
+    void getOrientedReadMarkerKmerIdsStrand1(ReadId, const span<KmerId>&) const;
+
+    // Get all MarkerWithOrdinals for an oriented read (includes position, KmerId, and ordinal).
+    void getOrientedReadMarkers(OrientedReadId, const span<MarkerWithOrdinal>&) const;
+    void getOrientedReadMarkersStrand0(ReadId, const span<MarkerWithOrdinal>&) const;
+    void getOrientedReadMarkersStrand1(ReadId, const span<MarkerWithOrdinal>&) const;
+
+    // Get all marker Kmers/KmerIds for a read in both orientations.
+    void getReadMarkerKmers(
+        ReadId,
+        const span<Kmer>& Kmers0,
+        const span<Kmer>& Kmers1) const;
+    void getReadMarkerKmerIds(
+        ReadId,
+        const span<KmerId>& kmerIds0,
+        const span<KmerId>& kmerIds1) const;
+
+    // Get the Kmer/KmerId for an oriented read at a given marker ordinal.
+    Kmer getOrientedReadMarkerKmer(OrientedReadId, uint64_t ordinal) const;
+    KmerId getOrientedReadMarkerKmerId(OrientedReadId, uint64_t ordinal) const;
 
 
 
@@ -698,16 +623,6 @@ public:
         size_t minFrequency,            // Minimum number of lowHash hits for a pair to become a candidate.
         size_t threadCount
     );
-    void findAlignmentCandidatesLowHash1(
-        size_t m,                       // Number of consecutive k-mers that define a feature.
-        double hashFraction,            // Low hash threshold.
-        size_t minHashIterationCount,
-        size_t log2MinHashBucketCount,  // Base 2 log of number of buckets for lowHash.
-        size_t minBucketSize,           // The minimum size for a bucket to be used.
-        size_t maxBucketSize,           // The maximum size for a bucket to be used.
-        size_t minFrequency,            // Minimum number of lowHash hits for a pair to become a candidate.
-        size_t threadCount
-    );
     void markAlignmentCandidatesAllPairs();
     void accessAlignmentCandidates();
     void accessAlignmentCandidateTable();
@@ -850,7 +765,7 @@ public:
         uint64_t maxBand,
         int64_t matchScore,
         int64_t mismatchScore,
-        int64_t gapScore) const;
+        int64_t gapScore);
 
     // Align two reads using alignment method 4.
     // If debug is true, detailed output to html is produced.
@@ -862,7 +777,7 @@ public:
         MemoryMapped::ByteAllocator&,
         Alignment&,
         AlignmentInfo&,
-        bool debug) const;
+        bool debug);
 
     // Intermediate level version used by the http server.
     void alignOrientedReads4(
@@ -883,7 +798,20 @@ public:
         int64_t gapScore,
         Alignment&,
         AlignmentInfo&
-        ) const;
+        );
+
+    // Alignment method 5.
+    void alignOrientedReads5(
+        OrientedReadId,
+        OrientedReadId,
+        int matchScore,
+        int mismatchScore,
+        int gapScore,
+        double driftRateTolerance,
+        uint64_t minBandExtend,
+        Alignment&,
+        AlignmentInfo&,
+        ostream& html);
 
 private:
 
@@ -939,7 +867,9 @@ private:
     // The good alignments we found.
     // They are stored with readId0<readId1 and with strand0==0.
     // The order in compressedAlignments matches that in alignmentData.
+public:
     MemoryMapped::Vector<AlignmentData> alignmentData;
+private:
     MemoryMapped::VectorOfVectors<char, uint64_t> compressedAlignments;
 
     void checkAlignmentDataAreOpen() const;
@@ -1011,8 +941,8 @@ private:
 
     // Read graph and related functions and data.
     // For more information, see comments in ReadGraph.hpp.
-    ReadGraph readGraph;
 public:
+    ReadGraph readGraph;
     void createReadGraph(
         uint32_t maxAlignmentCount,
         uint32_t maxTrim);
@@ -1291,6 +1221,7 @@ public:
     void accessMarkerGraphVertices(bool readWriteAccess = false);
     void accessMarkerGraphReverseComplementVertex(bool readWriteAccess = false);
     void removeMarkerGraphVertices();
+    void accessDisjointSetsHistogram();
 private:
     void findMarkerGraphReverseComplementVerticesThreadFunction1(size_t threadId);
     void findMarkerGraphReverseComplementVerticesThreadFunction2(size_t threadId);
@@ -1309,7 +1240,8 @@ private:
         uint32_t maxSkip,
         vector<MarkerGraphVertexId>&) const;
 
-
+    // Find the common KmerId for all the markers of a marker graph vertex.
+    KmerId getMarkerGraphVertexKmerId(MarkerGraphVertexId) const;
 
     // Clean up marker graph vertices that have duplicate markers
     // (more than one marker on the same oriented reads).
@@ -1461,6 +1393,38 @@ public:
 
 
 
+    // Analyze and compare the read compositions of two marker graph edges.
+    // This can only be done if the two edges have no duplicate OrientedReadIds
+    // in the markers. In that case, each OrientedReadId of an edge
+    // corresponds to one and only one markerInterval for each edge.
+    bool analyzeMarkerGraphEdgePair(
+        MarkerGraphEdgeId,
+        MarkerGraphEdgeId,
+        MarkerGraphEdgePairInfo&
+        ) const;
+    void writeHtmlMarkerGraphEdgePairInfo(
+        ostream& html,
+        MarkerGraphEdgeId,
+        MarkerGraphEdgeId,
+        const MarkerGraphEdgePairInfo&
+        ) const;
+
+    // Count the number of common oriented reads between two marker graph edges.
+    // This assumes, WITHOUT CHECKING, that each of the two edges has no duplicate
+    // oriented reads. This assumption is satisfied for primary marker graph edges
+    // in Mode 3 assembly.
+    uint64_t countCommonOrientedReadsUnsafe(MarkerGraphEdgeId, MarkerGraphEdgeId) const;
+
+    // Estimate the offset, in bases, between two marker graph edges.
+    // This assumes, WITHOUT CHECKING, that each of the two edges has no duplicate
+    // oriented reads. This assumption is satisfied for primary marker graph edges
+    // in Mode 3 assembly.
+    // If there are common oriented reads between the two edges, this uses
+    // countCommonOrientedReadsUnsafe.
+    // This can fail, in which case it returns invalid<uint64_t>.
+    uint64_t estimateBaseOffsetUnsafe(MarkerGraphEdgeId, MarkerGraphEdgeId) const;
+
+
     // Function createMarkerGraphSecondaryEdges can be called after createMarkerGraphEdgesStrict
     // to create a minimal amount of additional non-strict edges (secondary edges)
     // sufficient to restore contiguity.
@@ -1563,9 +1527,11 @@ private:
     // it belongs to, plus the ordinal of the marker in the oriented read.
     // If the marker is not contained in any vertex, return
     // MarkerGraph::invalidVertexId.
+public:
     MarkerGraph::VertexId getGlobalMarkerGraphVertex(
         OrientedReadId,
         uint32_t ordinal) const;
+private:
 
     // Get pairs (ordinal, marker graph vertex id) for all markers of an oriented read.
     // The pairs are returned sorted by ordinal.
@@ -1711,7 +1677,7 @@ private:
         bool useSuperBubbleEdges,
         bool useLowCoverageCrossEdges,
         bool useRemovedSecondaryEdges,
-        LocalMarkerGraph&
+        LocalMarkerGraph0&
         );
     bool extractLocalMarkerGraph(
         MarkerGraph::VertexId,
@@ -1724,7 +1690,7 @@ private:
         bool useSuperBubbleEdges,
         bool useLowCoverageCrossEdges,
         bool useRemovedSecondaryEdges,
-        LocalMarkerGraph&
+        LocalMarkerGraph0&
         );
 
     // Compute consensus sequence for a vertex of the marker graph.
@@ -1777,20 +1743,6 @@ private:
 
 
 
-    // Get the RLE sequence implied by a MarkerInterval.
-    // If the markers overlap, returns the number of
-    // overlapping RLE bases in overlappingRleBaseCount
-    // and empty rleSequence.
-    // Otherwise, returns zero overlappingRleBaseCount
-    // and the intervening sequence in rleSequence
-    // (which can be empty if the two markers are exactly adjacent).
-    void getMarkerIntervalRleSequence(
-        const MarkerInterval&,
-        uint64_t& overlappingRleBaseCount,
-        vector<Base>& rleSequence) const;
-
-
-
     // Use spoa to compute consensus sequence for an edge of the marker graph.
     // This does not include the bases corresponding to the flanking markers.
     void computeMarkerGraphEdgeConsensusSequenceUsingSpoa(
@@ -1840,7 +1792,7 @@ public:
     // A directed vertex A->B is created if the last marker graph vertex
     // of the edge chain corresponding to A coincides with the
     // first marker graph vertex of the edge chain corresponding to B.
-    shared_ptr<AssemblyGraph> assemblyGraphPointer;
+    shared_ptr<mode0::AssemblyGraph> assemblyGraphPointer;
     void removeAssemblyGraph()
     {
         assemblyGraphPointer.reset();
@@ -2045,7 +1997,17 @@ public:
 private:
 
 
+
+    // Assemble Mode 3 sequence for all marker graph edges.
+    // See the comments before MarkerGraph::edgeSequence for more information.
+    // For now this is done sequentially.
+public:
+    void assembleMarkerGraphEdgesMode3();
+
+
+
     // Assemble sequence for an edge of the assembly graph.
+private:
     void assembleAssemblyGraphEdge(
         AssemblyGraphEdgeId,
         bool storeCoverageData,
@@ -2085,24 +2047,9 @@ private:
 
 
 
-    // Assemble the RLE sequence of a path of the marker graph, under the assumption
-    // that, for each edge, all oriented reads have exactly the same sequence.
-    // This will be the case if edges were created by Assembler::createMarkerGraphEdgesStrict.
-public:
-    void assembleMarkerGraphPathRleStrict(
-        span<const MarkerGraphEdgeId> path,
-        vector<Base>& rleSequence
-    ) const;
-    // Same, but for an assembly graph edge.
-    void assembleAssemblyGraphEdgeRleStrict(
-        AssemblyGraphEdgeId,
-        vector<Base>& rleSequence
-    ) const;
-
-
-
     // Write the assembly graph in GFA 1.0 format defined here:
     // https://github.com/GFA-spec/GFA-spec/blob/master/GFA1.md
+public:
     void writeGfa1(const string& fileName);
     void writeGfa1BothStrands(const string& fileName);
     void writeGfa1BothStrandsNoSequence(const string& fileName);
@@ -2151,11 +2098,6 @@ public:
 
 
 
-    // Analyze pseudo-paths of oriented reads.
-    void alignPseudoPaths(ReadId, Strand, ReadId, Strand);
-
-
-
     // Data and functions used for the http server.
     // This function puts the server into an endless loop
     // of processing requests.
@@ -2247,12 +2189,14 @@ public:
 
     // Functions and data used by the http server
     // for display of the local marker graph.
-    void exploreMarkerGraph(const vector<string>&, ostream&);
-    void getLocalMarkerGraphRequestParameters(
+    void exploreMarkerGraph0(const vector<string>&, ostream&);
+    void exploreMarkerGraph1(const vector<string>&, ostream&);
+    void getLocalMarkerGraph0RequestParameters(
         const vector<string>&,
-        LocalMarkerGraphRequestParameters&) const;
+        LocalMarkerGraph0RequestParameters&) const;
     void exploreMarkerGraphVertex(const vector<string>&, ostream&);
     void exploreMarkerGraphEdge(const vector<string>&, ostream&);
+    void exploreMarkerGraphEdgePair(const vector<string>&, ostream&);
     void exploreMarkerCoverage(const vector<string>&, ostream&);
     void exploreMarkerGraphInducedAlignment(const vector<string>&, ostream&);
     void followReadInMarkerGraph(const vector<string>&, ostream&);
@@ -2275,6 +2219,8 @@ public:
         uint64_t align4DeltaY,
         uint64_t align4MinEntryCountPerCell,
         uint64_t align4MaxDistanceFromBoundary,
+        double align5DriftRateTolerance,
+        uint64_t align5MinBandExtend,
         ostream& html
     );
     void writeColorPicker(ostream& html, string svgId);
@@ -2305,7 +2251,7 @@ public:
 
     // Compute all alignments for a given read.
     // This can be slow for large assemblies,
-    // and therefore the computation in multithreaded.
+    // and therefore the computation is multithreaded.
     void computeAllAlignments(const vector<string>&, ostream&);
     void computeAllAlignmentsThreadFunction(size_t threadId);
     class ComputeAllAlignmentsData {
@@ -2329,6 +2275,8 @@ public:
         uint64_t align4DeltaY;
         uint64_t align4MinEntryCountPerCell;
         uint64_t align4MaxDistanceFromBoundary;
+        double align5DriftRateTolerance;
+        uint64_t align5MinBandExtend;
         // The alignments found by each thread.
         vector< vector< pair<OrientedReadId, AlignmentInfo> > > threadAlignments;
     };
@@ -2374,16 +2322,6 @@ private:
     void exploreAssemblyGraphEdgesSupport(const vector<string>&, ostream&);
 
 
-    // Http server functionality specific to mode 3 assembly.
-    void exploreMode3AssemblyGraph(const vector<string>&, ostream&);
-    void exploreMode3AssemblyGraphSegment(const vector<string>&, ostream&);
-    void exploreMode3AssemblyGraphSegmentPair(const vector<string>&, ostream&);
-    void exploreMode3AssemblyGraphLink(const vector<string>&, ostream&);
-    void exploreMode3MetaAlignment(const vector<string>&, ostream&);
-    void exploreMode3AssemblyPath(const vector<string>&, ostream&);
-    void exploreMode3LinkAssembly(const vector<string>&, ostream&);
-
-
 
     // Set up the ConsensusCaller used to compute the "best"
     // base and repeat count at each assembly position.
@@ -2415,16 +2353,30 @@ public:
         bool debug);
 
 
+
     // Mode 3 assembly.
+    shared_ptr<Mode3Assembler> mode3Assembler;
+    void flagPrimaryMarkerGraphEdges(
+        uint64_t minPrimaryCoverage,
+        uint64_t maxPrimaryCoverage,
+        uint64_t threadCount);
+
+    // Assemble sequence between two primary edges.
+    void fillMode3AssemblyPathStep(const vector<string>&, ostream&);
+
+    // Top level function for Mode 3 assembly, starting from the MarkerGraph.
     void mode3Assembly(
-        size_t threadCount);
-    shared_ptr<mode3::AssemblyGraph> assemblyGraph3Pointer;
-    void accessMode3AssemblyGraph();
-    void analyzeMode3Subgraph(const vector<uint64_t>& segmentIds);
-    void createMode3PathGraph();
-    void createMode3Detangler();
-
+        uint64_t threadCount,
+        const Mode3AssemblyOptions&,
+        bool debug
+    );
 
+    void mode3AssembleComponent(
+        const string& fileName,
+        uint64_t threadCount,
+        const Mode3AssemblyOptions&,
+        bool assembleSequence,
+        bool debug) const;
 
 public:
     void test();
