@@ -214,13 +214,14 @@ void Assembler::computeAlignments(
     size_t threadCount
 )
 {
+
     const auto tBegin = steady_clock::now();
     performanceLog << timestamp << "Begin computing alignments for ";
     performanceLog << alignmentCandidates.candidates.size() << " alignment candidates." << endl;
 
     // Check that we have what we need.
     reads->checkReadsAreOpen();
-    checkKmersAreOpen();
+    SHASTA_ASSERT(kmerChecker);
     checkMarkersAreOpen();
     checkAlignmentCandidatesAreOpen();
 
@@ -239,6 +240,12 @@ void Assembler::computeAlignments(
         computeSortedMarkers(threadCount);
     }
 
+    // For alignment method 5, compute low frequency markers.
+    if(alignOptions.alignMethod == 5) {
+        cout << timestamp << "Computing unique markers." << endl;
+        computeLowFrequencyMarkers(1, threadCount);
+    }
+
     // Pick the batch size for computing alignments.
     size_t batchSize = 10;
     if(batchSize > alignmentCandidates.candidates.size()/threadCount) {
@@ -254,9 +261,11 @@ void Assembler::computeAlignments(
     data.threadCompressedAlignments.resize(threadCount);
     
     performanceLog << timestamp << "Alignment computation begins." << endl;
+    cout << timestamp << "Alignment computation begins." << endl;
     setupLoadBalancing(alignmentCandidates.candidates.size(), batchSize);
     runThreads(&Assembler::computeAlignmentsThreadFunction, threadCount);
     performanceLog << timestamp << "Alignment computation completed." << endl;
+    cout << timestamp << "Alignment computation completed." << endl;
 
     // Store the alignments found by each thread.
     performanceLog << timestamp << "Storing the alignment found by each thread." << endl;
@@ -286,10 +295,13 @@ void Assembler::computeAlignments(
     alignmentData.unreserve();
     compressedAlignments.unreserve();
 
-    // For alignment method 4, remove the sorted markers.
+    // Cleanup.
     if(alignOptions.alignMethod == 4) {
         sortedMarkers.remove();
     }
+    if(alignOptions.alignMethod == 5) {
+        lowFrequencyMarkers.remove();
+    }
 
     cout << "Found and stored " << alignmentData.size() << " good alignments." << endl;
     performanceLog << timestamp << "Creating alignment table." << endl;
@@ -331,6 +343,8 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
     const int bandExtend = data.alignOptions->bandExtend;
     const int maxBand = data.alignOptions->maxBand;
     const bool suppressContainments = data.alignOptions->suppressContainments;
+    const double align5DriftRateTolerance = data.alignOptions->align5DriftRateTolerance;
+    const uint64_t align5MinBandExtend = data.alignOptions->align5MinBandExtend;
 
 
     // Align4-specific items.
@@ -365,13 +379,10 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
         largeDataName("tmp-ThreadGlobalCompressedAlignments-" + to_string(threadId)),
         largeDataPageSize);
 
+    const uint64_t messageFrequency = min(1000000UL, alignmentCandidates.candidates.size()/20);
+
     uint64_t begin, end;
     while(getNextBatch(begin, end)) {
-        if((begin % 1000000) == 0){
-            std::lock_guard<std::mutex> lock(mutex);
-            performanceLog << timestamp << "Working on alignment " << begin;
-            performanceLog << " of " << alignmentCandidates.candidates.size() << endl;
-        }
 
         for(size_t i=begin; i!=end; i++) {
             const OrientedReadPair& candidate = alignmentCandidates.candidates[i];
@@ -381,6 +392,13 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
             orientedReadIds[0] = OrientedReadId(candidate.readIds[0], 0);
             orientedReadIds[1] = OrientedReadId(candidate.readIds[1], candidate.isSameStrand ? 0 : 1);
 
+            if((i % messageFrequency) == 0){
+                std::lock_guard<std::mutex> lock(mutex);
+                performanceLog << timestamp << "Working on alignment " << i;
+                performanceLog << " of " << alignmentCandidates.candidates.size();
+                // performanceLog << ": " << orientedReadIds[0] << " " << orientedReadIds[1];
+                performanceLog << endl;
+            }
 
 
             // Compute the alignment.
@@ -413,6 +431,13 @@ void Assembler::computeAlignmentsThreadFunction(size_t threadId)
                         alignment, alignmentInfo,
                         false);
                     SHASTA_ASSERT(byteAllocator.isEmpty());
+                } else if(alignmentMethod == 5) {
+                    ofstream nullStream;
+                    alignOrientedReads5(orientedReadIds[0], orientedReadIds[1],
+                        matchScore, mismatchScore, gapScore,
+                        align5DriftRateTolerance, align5MinBandExtend,
+                        alignment, alignmentInfo,
+                        nullStream);
                 } else {
                     SHASTA_ASSERT(0);
                 }
@@ -1008,70 +1033,6 @@ uint32_t Assembler::countCommonMarkersWithOffsetIn(
 }
 
 
-void Assembler::writeAlignmentDetails() const
-{
-    string directoryName = "Alignments/";
-    string header = "kmerId,ordinal0,ordinal1,rlePosition0,rlePosition1,";
-
-    SHASTA_ASSERT(std::filesystem::create_directory(directoryName));
-
-    for (uint32_t alignmentIndex=0; alignmentIndex<alignmentData.size(); alignmentIndex++){
-        // Access the stored information we have about this alignment.
-        AlignmentData alignmentDatum = alignmentData[alignmentIndex];
-        span<const char> compressedAlignment = compressedAlignments[alignmentIndex];
-
-        Alignment alignment;
-        decompress(compressedAlignment, alignment);
-        OrientedReadId orientedReadId0 = OrientedReadId(alignmentDatum.readIds[0], 0);
-        OrientedReadId orientedReadId1 = OrientedReadId(alignmentDatum.readIds[1], alignmentDatum.isSameStrand ? 0 : 1);
-
-        string name0 = string(reads->getReadName(orientedReadId0.getReadId()).begin(),
-                              reads->getReadName(orientedReadId0.getReadId()).end());
-
-        string name1 = string(reads->getReadName(orientedReadId1.getReadId()).begin(),
-                              reads->getReadName(orientedReadId1.getReadId()).end());
-
-        string filename = name0 + "_" + name1 + "_" + (alignmentDatum.isSameStrand ? "1" : "0") + ".csv";
-
-        // Create a writeable a csv file
-        ofstream csv(directoryName + filename);
-        if (not (csv.is_open() and csv.good())){
-            throw runtime_error("ERROR: file could not be written: " + directoryName + filename);
-        }
-
-        csv << header << '\n';
-
-        // Access the markers for the two oriented reads.
-        const auto markers0 = markers[orientedReadId0.getValue()];
-        const auto markers1 = markers[orientedReadId1.getValue()];
-
-        // Compute the raw position corresponding to each RLE position.
-        const vector<uint32_t> rawPositions0 = reads->getRawPositions(orientedReadId0);
-        const vector<uint32_t> rawPositions1 = reads->getRawPositions(orientedReadId1);
-
-        // Loop over all markers.
-        for(const auto& ordinals: alignment.ordinals) {
-            const auto ordinal0 = ordinals[0];
-            const auto ordinal1 = ordinals[1];
-
-            const auto& marker0 = markers0[ordinal0];
-            const auto& marker1 = markers1[ordinal1];
-
-            const uint32_t rlePosition0 = marker0.position;
-            const uint32_t rlePosition1 = marker1.position;
-
-            const auto kmerId = marker0.kmerId;
-            SHASTA_ASSERT(marker1.kmerId == kmerId);
-
-            csv << kmerId << ','
-                << ordinal0 << ','
-                << ordinal1 << ','
-                << rlePosition0 << ','
-                << rlePosition1 << ',' << '\n';
-        }
-    }
-}
-
 
 // Check if an alignment between two reads should be suppressed,
 // bases on the setting of command line option
