@@ -6,15 +6,15 @@
 #include "seqan.hpp"
 #include "timestamp.hpp"
 using namespace shasta;
+using namespace mode0;
 
 // Standard library.
 #include "fstream.hpp"
 
 
 
-// This use PseudoPaths to decide which alignments
+// This uses PseudoPaths to decide which alignments
 // should be included in the read graph.
-// See Assembler::alignPseudoPaths in AssemblerAnalyzePaths.cpp.
 void Assembler::createReadGraphUsingPseudoPaths(
     int64_t matchScore,
     int64_t mismatchScore,
