@@ -654,6 +654,8 @@ void Assembler::exploreReadRle(
                 if (marker.position < beginRlePosition || marker.position > endRlePosition-k) {
                     continue;
                 }
+                const Kmer kmer = getOrientedReadMarkerKmer(orientedReadId, ordinal);
+                const KmerId kmerId = KmerId(kmer.id(k));
 
                 // See if this marker is contained in a vertex of the marker graph.
                 const MarkerGraph::VertexId vertexId =
@@ -664,10 +666,9 @@ void Assembler::exploreReadRle(
 
 
                 // Write the k-mer of this marker.
-                const Kmer kmer(marker.kmerId, k);
                 html << "<a xlink:title='Marker " << ordinal <<
                     ", position " << marker.position <<
-                    ", k-mer id " << marker.kmerId;
+                    ", k-mer id " << kmerId;
                 if(hasMarkerGraphVertex) {
                     html << ", coverage " << markerGraph.vertexCoverage(vertexId);
                 }
@@ -675,7 +676,7 @@ void Assembler::exploreReadRle(
                 if(hasMarkerGraphVertex) {
                     // Add a hyperlink to the marker graph vertex
                     // that contains this marker.
-                    const string url = "exploreMarkerGraph?vertexId=" + to_string(vertexId) +
+                    const string url = "exploreMarkerGraph0?vertexId=" + to_string(vertexId) +
                         "&maxDistance=2&detailed=on&minCoverage=3&minConsensus=3&sizePixels=320&timeout=30";
                     html << " xlink:href='" << url << "' style='cursor:pointer'";
                 }
@@ -762,7 +763,7 @@ void Assembler::exploreReadRle(
         // Loop over all markers on this oriented read.
         for(uint32_t ordinal=0; ordinal<orientedReadMarkers.size(); ordinal++) {
             const CompressedMarker& marker = orientedReadMarkers[ordinal];
-            const Kmer kmer(marker.kmerId, k);
+            const Kmer kmer = getOrientedReadMarkerKmer(orientedReadId, ordinal);
             const uint32_t rlePosition = marker.position;
             const uint32_t rawPosition = rawPositions[rlePosition];
 
@@ -783,7 +784,7 @@ void Assembler::exploreReadRle(
             html << "</code><td class=centered>" << rlePosition << "<td class=centered>" << rawPosition;
 
             if(hasMarkerGraphVertex) {
-                const string url = "exploreMarkerGraph?vertexId=" + to_string(vertexId) +
+                const string url = "exploreMarkerGraph0?vertexId=" + to_string(vertexId) +
                     "&maxDistance=2&detailed=on&minCoverage=3&minConsensus=3&sizePixels=320&timeout=30";
                 html << "<td class=centered><a href='" << url << "'>" << vertexId << "</a>"
                     "<td class=centered>" << markerGraph.vertexCoverage(vertexId);
@@ -806,7 +807,7 @@ void Assembler::exploreReadRle(
         for(uint32_t ordinal=0; ordinal<uint32_t(orientedReadMarkers.size()); ordinal++) {
             const CompressedMarker& marker = orientedReadMarkers[ordinal];
             if (marker.position >= beginRlePosition && marker.position <= endRlePosition - k) {
-                kmers.push_back(marker.kmerId);
+                kmers.push_back(getOrientedReadMarkerKmerId(orientedReadId, ordinal));
             }
         }
         vector<uint32_t> kmerFrequency;
@@ -1081,10 +1082,10 @@ void Assembler::exploreReadRaw(
         for(const uint64_t ordinal: markersOnThisRow) {
             const CompressedMarker& marker = orientedReadMarkers[ordinal];
             const uint64_t position = marker.position - beginPosition;
-            const Kmer kmer(marker.kmerId, k);
+            const Kmer kmer = getOrientedReadMarkerKmer(orientedReadId, ordinal);
 
             // Write the required number of spaces.
-            SHASTA_ASSERT(position > oldPosition);  // There must be at least a blank.
+            SHASTA_ASSERT((position==0) or (position > oldPosition));  // There must be at least a blank.
             for(uint64_t i=oldPosition; i<position; i++) {
                 html << "&nbsp;";
             }
@@ -1100,7 +1101,7 @@ void Assembler::exploreReadRaw(
 
                 // There is a marker graph vertex.
                 // Write the marker as a link to that vertex.
-                const string url = "exploreMarkerGraph?vertexId=" + to_string(vertexId) +
+                const string url = "exploreMarkerGraph0?vertexId=" + to_string(vertexId) +
                     "&maxDistance=6&detailed=on&sizePixels=600&timeout=30";
                 html << "<a href='" << url << "' title='Marker " << ordinal <<
                     ", position " << marker.position <<
