@@ -8,6 +8,7 @@
 #include "platformDependent.hpp"
 #include "Reads.hpp"
 using namespace shasta;
+using namespace mode0;
 
 // Boost libraries.
 #include <boost/tokenizer.hpp>
@@ -230,9 +231,11 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(alignSequencesInMarkerRepresentation);
     SHASTA_ADD_TO_FUNCTION_TABLE(assessAlignments);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreReadGraph);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraph);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraph0);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraph1);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphVertex);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphEdge);
+    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphEdgePair);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerCoverage);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraphInducedAlignment);
     SHASTA_ADD_TO_FUNCTION_TABLE(followReadInMarkerGraph);
@@ -241,14 +244,8 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdge);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAssemblyGraphEdgesSupport);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreCompressedAssemblyGraph);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraph);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphSegment);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphSegmentPair);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyGraphLink);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3MetaAlignment);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3AssemblyPath);
-    SHASTA_ADD_TO_FUNCTION_TABLE(exploreMode3LinkAssembly);
 
+    SHASTA_ADD_TO_FUNCTION_TABLE(fillMode3AssemblyPathStep);
 }
 #undef SHASTA_ADD_TO_FUNCTION_TABLE
 
@@ -442,15 +439,37 @@ void Assembler::writeNavigation(ostream& html) const
     writeNavigation(html, "Read graph", {
         {"Read graph", "exploreReadGraph"},
         });
-    writeNavigation(html, "Marker graph", {
-        {"Local marker graph", "exploreMarkerGraph?useBubbleReplacementEdges=on"},
-        {"Marker graph vertices", "exploreMarkerGraphVertex"},
-        {"Marker graph edges", "exploreMarkerGraphEdge"},
-        {"Marker coverage", "exploreMarkerCoverage"},
-        {"Induced alignments", "exploreMarkerGraphInducedAlignment"},
-        {"Follow a read in the marker graph", "followReadInMarkerGraph"},
-        {"Marker connectivity", "exploreMarkerConnectivity"},
-        });
+
+
+
+    if(assemblerInfo->assemblyMode == 3) {
+        writeNavigation(html, "Marker graph", {
+            {"Local marker graph", "exploreMarkerGraph0?useBubbleReplacementEdges=on"},
+            {"Local marker graph for mode 3 assembly", "exploreMarkerGraph1"},
+            {"Marker graph vertices", "exploreMarkerGraphVertex"},
+            {"Marker graph edges", "exploreMarkerGraphEdge"},
+            {"Marker graph edge pairs", "exploreMarkerGraphEdgePair"},
+            {"Marker coverage", "exploreMarkerCoverage"},
+            {"Induced alignments", "exploreMarkerGraphInducedAlignment"},
+            {"Follow a read in the marker graph", "followReadInMarkerGraph"},
+            {"Marker connectivity", "exploreMarkerConnectivity"},
+            {"Assembly path step", "fillMode3AssemblyPathStep"},
+            {"Path graph", "exploreMode3PathGraph"},
+            });
+    } else {
+        writeNavigation(html, "Marker graph", {
+            {"Local marker graph", "exploreMarkerGraph0?useBubbleReplacementEdges=on"},
+            {"Marker graph vertices", "exploreMarkerGraphVertex"},
+            {"Marker graph edges", "exploreMarkerGraphEdge"},
+            {"Marker coverage", "exploreMarkerCoverage"},
+            {"Induced alignments", "exploreMarkerGraphInducedAlignment"},
+            {"Follow a read in the marker graph", "followReadInMarkerGraph"},
+            {"Marker connectivity", "exploreMarkerConnectivity"},
+            });
+    }
+
+
+
     if(assemblerInfo->assemblyMode == 0) {
         writeNavigation(html, "Assembly graph", {
             {"Local assembly graph", "exploreAssemblyGraph"},
@@ -459,16 +478,6 @@ void Assembler::writeNavigation(ostream& html) const
             {"Compressed assembly graph", "exploreCompressedAssemblyGraph"},
             });
     }
-    if(assemblerInfo->assemblyMode == 3) {
-        writeNavigation(html, "Assembly graph", {
-            {"Local assembly graph", "exploreMode3AssemblyGraph"},
-            {"Assembly graph segments", "exploreMode3AssemblyGraphSegment"},
-            {"Assembly graph segment pairs", "exploreMode3AssemblyGraphSegmentPair"},
-            {"Assembly graph links", "exploreMode3AssemblyGraphLink"},
-            {"Meta-alignments", "exploreMode3MetaAlignment"},
-            {"Assembly paths", "exploreMode3AssemblyPath"},
-            });
-    }
 
     if (!httpServerData.docsDirectory.empty()) {
         writeNavigation(html, "Help", {
@@ -566,6 +575,10 @@ void Assembler::writeGnuPlotPngToHtml(
 
     // Write the png file to html.
     writePngToHtml(html, pngFileName);
+
+    // Remove the files we created.
+    std::filesystem::remove(gnuplotFileName);
+    std::filesystem::remove(pngFileName);
 }
 
 
@@ -586,9 +599,9 @@ void Assembler::accessAllSoft()
     bool allDataAreAvailable = true;
 
     try {
-        accessKmers();
+        accessKmerChecker();
     } catch(const exception& e) {
-        cout << "K-mers are not accessible." << endl;
+        cout << "The k-mer checker is not accessible." << endl;
         allDataAreAvailable = false;
     }
 
@@ -675,6 +688,17 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
+#if 0
+    if(assemblerInfo->assemblyMode == 3) {
+        try {
+            accessMarkerGraphPrimaryJourneys();
+        } catch(const exception& e) {
+            cout << "MarkerGraph graph primary journeys are not accessible." << endl;
+            allDataAreAvailable = false;
+        }
+    }
+#endif
+
     try {
         accessCompressedAlignments();
     } catch(const exception& e) {
@@ -718,18 +742,6 @@ void Assembler::accessAllSoft()
 
 
 
-    // Data specific to assembly mode 3.
-    if(assemblerInfo->assemblyMode == 3) {
-        try {
-            accessMode3AssemblyGraph();
-        } catch(const exception& e) {
-            cout << "The mode 3 assembly graph is not accessible." << endl;
-            allDataAreAvailable = false;
-        }
-    }
-
-
-
     if(!allDataAreAvailable) {
         cout << "Not all assembly data are accessible." << endl;
         cout << "Some functionality is not available." << endl;
@@ -906,20 +918,7 @@ void Assembler::writeAssemblySummary(ostream& html)
 void Assembler::writeAssemblySummaryBody(ostream& html)
 {
     using std::setprecision;
-    AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
-
-
-    // Compute the number of run-length k-mers used as markers.
-    uint64_t totalRleKmerCount = 0;
-    uint64_t markerRleKmerCount = 0;
-    for(const auto& tableEntry: kmerTable) {
-        if(tableEntry.isRleKmer) {
-            ++totalRleKmerCount;
-            if(tableEntry.isMarker) {
-                ++markerRleKmerCount;
-            }
-        }
-    }
+    mode0::AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
 
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
@@ -1006,21 +1005,6 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
 
 
     html <<
-        "<h3>Marker <i>k</i>-mers</h3>"
-        "<table>"
-        "<tr><td>Length <i>k</i> of <i>k</i>-mers used as markers"
-        "<td class=right>" << assemblerInfo->k <<
-        "<tr><td>Total number of <i>k</i>-mers"
-        "<td class=right>" << totalRleKmerCount <<
-        "<tr><td>Number of <i>k</i>-mers used as markers"
-        "<td class=right>" << markerRleKmerCount <<
-        "<tr><td>Fraction of <i>k</i>-mers used as markers"
-        "<td class=right>" << setprecision(3) << double(markerRleKmerCount) / double(totalRleKmerCount) <<
-        "</table>"
-        "<ul><li>In the above table, all <i>k</i>-mer counts only include run-length encoded <i>k</i>-mers, "
-        "that is, <i>k</i>-mers without repeated bases.</ul>"
-
-
 
         "<h3>Markers</h3>"
         "<table>"
@@ -1240,20 +1224,6 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
     AssemblyGraph& assemblyGraph = *assemblyGraphPointer;
     using std::setprecision;
 
-
-
-    // Compute the number of run-length k-mers used as markers.
-    uint64_t totalRleKmerCount = 0;
-    uint64_t markerRleKmerCount = 0;
-    for(const auto& tableEntry: kmerTable) {
-        if(tableEntry.isRleKmer) {
-            ++totalRleKmerCount;
-            if(tableEntry.isMarker) {
-                ++markerRleKmerCount;
-            }
-        }
-    }
-
     const uint64_t totalDiscardedReadCount =
         assemblerInfo->discardedInvalidBaseReadCount +
         assemblerInfo->discardedShortReadReadCount +
@@ -1332,17 +1302,6 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         double(totalDiscardedBaseCount + assemblerInfo->baseCount)
         << "\n"
         "    }\n"
-        "  },\n";
-
-
-    json <<
-        "  \"Marker k-mers\":\n"
-        "  {\n"
-        "    \"Length k of k-mers used as markers\": " << assemblerInfo->k << ",\n"
-        "    \"Total number of k-mers\": " << totalRleKmerCount << ",\n"
-        "    \"Number of k-mers used as markers\": " << markerRleKmerCount << ",\n"
-        "    \"Fraction of k<-mers used as markers\": " <<
-        setprecision(3) << double(markerRleKmerCount) / double(totalRleKmerCount) <<
         "  },\n"
 
 
