@@ -14,7 +14,11 @@ using namespace shasta;
 
 
 
-// Constructor.
+// Constructor from a command line.
+// If the command line includes a --config option,
+// the specified built-in configuration or configuration file
+// is used to fill the AssemblyOptions,
+// but values specified on the command line take precedence.
 AssemblerOptions::AssemblerOptions(int argumentCount, const char** arguments) :
     commandLineOnlyOptionsDescription("Options allowed only on the command line"),
     configurableOptionsDescription("Options allowed on the command line and in the config file")
@@ -124,6 +128,41 @@ AssemblerOptions::AssemblerOptions(int argumentCount, const char** arguments) :
 
 
 
+// Constructor from a configuration file.
+// This only fills in the configurable options specified in
+// the given configuration file. Command line only options
+// are left at their defaults.
+AssemblerOptions::AssemblerOptions(const string& fileName)
+{
+
+    using boost::program_options::positional_options_description;
+    using boost::program_options::value;
+    using boost::program_options::variables_map;
+    using boost::program_options::command_line_parser;
+
+    addConfigurableOptions();
+
+    ifstream configFile(fileName);
+    if (not configFile) {
+        throw runtime_error("Invalid configuration file " + fileName + " specified.\n");
+    }
+    variables_map variablesMap;
+    store(parse_config_file(configFile, configurableOptionsDescription), variablesMap);
+    notify(variablesMap);
+
+    // Parse MarkerGraph.simplifyMaxLength.
+    markerGraphOptions.parseSimplifyMaxLength();
+
+    // Parse ReadOptions.desiredCoverageString into its numeric value.
+    readsOptions.parseDesiredCoverageString();
+
+    // Unpack the consensuscaller and replace relative path with the absolute
+    // one if necessary.
+    assemblyOptions.parseConsensusCallerString();
+}
+
+
+
 // Add non-configurable options to the Boost option description object.
 // These are options that can only be used on the command line
 // (not in the configuration file).
@@ -252,6 +291,12 @@ void AssemblerOptions::addConfigurableOptions()
         "This is done by specifying the O_DIRECT flag when opening "
         "input files containing reads.")
 
+        ("Reads.handleDuplicates",
+        value<string>(&readsOptions.handleDuplicates)->
+        default_value("useOneCopy"),
+        "Controls handling of reads with duplicate names. "
+        "Can be one of: useAllCopies, useOneCopy, useNone, forbid.")
+
         ("Reads.palindromicReads.skipFlagging",
         bool_switch(&readsOptions.palindromicReads.skipFlagging)->
         default_value(false),
@@ -327,8 +372,7 @@ void AssemblerOptions::addConfigurableOptions()
         ("MinHash.version",
         value<int>(&minHashOptions.version)->
         default_value(0),
-        "Controls the version of the LowHash algorithm to use. Can be 0 (default) "
-        "or 1.(experimental).")
+        "Controls the version of the LowHash algorithm to use. Must be 0 (default).")
 
         ("MinHash.m",
         value<int>(&minHashOptions.m)->
@@ -357,12 +401,16 @@ void AssemblerOptions::addConfigurableOptions()
         ("MinHash.minBucketSize",
         value<int>(&minHashOptions.minBucketSize)->
         default_value(0),
-        "The minimum bucket size to be used by the LowHash algorithm.")
+        "The minimum bucket size to be used by the LowHash algorithm. "
+        "If minBucketSize and maxBucketSize are both 0, they are adjusted automatically "
+        "at each iteration using simple heuristics.")
 
         ("MinHash.maxBucketSize",
         value<int>(&minHashOptions.maxBucketSize)->
         default_value(10),
-        "The maximum bucket size to be used by the LowHash algorithm.")
+        "The maximum bucket size to be used by the LowHash algorithm. "
+        "If minBucketSize and maxBucketSize are both 0, they are adjusted automatically "
+        "at each iteration using simple heuristics.")
 
         ("MinHash.minFrequency",
         value<int>(&minHashOptions.minFrequency)->
@@ -381,7 +429,7 @@ void AssemblerOptions::addConfigurableOptions()
         value<int>(&alignOptions.alignMethod)->
         default_value(3),
         "The alignment method to be used to create the read graph & the marker graph. "
-        "0 = old Shasta method, 1 = SeqAn (slow), 3 = banded SeqAn, 4 = new Shasta method (experimental).")
+        "0 = old Shasta method, 1 = SeqAn (slow), 3 = banded SeqAn, 4 and 5 = experimental.")
 
         ("Align.maxSkip",
         value<int>(&alignOptions.maxSkip)->
@@ -488,6 +536,16 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(100),
         "Only used for alignment method 4 (experimental).")
 
+        ("Align.align5.driftRateTolerance",
+        value<double>(&alignOptions.align5DriftRateTolerance)->
+        default_value(0.02),
+        "Maximum allowed drift rate for alignment method 5.")
+
+        ("Align.align5.minBandExtend",
+        value<uint64_t>(&alignOptions.align5MinBandExtend)->
+        default_value(10),
+        "Minimum band extension for alignment method 5.")
+
         ("ReadGraph.creationMethod",
         value<int>(&readGraphOptions.creationMethod)->
         default_value(0),
@@ -501,7 +559,7 @@ void AssemblerOptions::addConfigurableOptions()
         ("ReadGraph.maxChimericReadDistance",
         value<int>(&readGraphOptions.maxChimericReadDistance)->
         default_value(2),
-        "Used for chimeric read detection.")
+        "Used for chimeric read detection. Set to 0 to turn off chimera detection.")
 
         ("ReadGraph.strandSeparationMethod",
         value<uint64_t>(&readGraphOptions.strandSeparationMethod)->
@@ -599,7 +657,7 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(6),
         "Minimum edge coverage (number of supporting oriented reads) "
         "for a marker graph edge to be created."
-        "Experimental. Only used with --Assembly.mode 1.")
+        "Only used with --Assembly.mode 2.")
 
         ("MarkerGraph.minEdgeCoveragePerStrand",
         value<uint64_t>(&markerGraphOptions.minEdgeCoveragePerStrand)->
@@ -607,7 +665,7 @@ void AssemblerOptions::addConfigurableOptions()
         "Minimum edge coverage (number of supporting oriented reads) "
         "on each strand "
         "for a marker graph edge to be created."
-        "Experimental. Only used with --Assembly.mode 1.")
+        "Only used with --Assembly.mode 2.")
 
         ("MarkerGraph.allowDuplicateMarkers",
         bool_switch(&markerGraphOptions.allowDuplicateMarkers)->
@@ -667,11 +725,6 @@ void AssemblerOptions::addConfigurableOptions()
         "corresponding marker graph edges. A cross edge is defined as an edge v0->v1 "
         "with out-degree(v0)>1, in-degree(v1)>1.")
 
-        ("MarkerGraph.reverseTransitiveReduction",
-        bool_switch(&markerGraphOptions.reverseTransitiveReduction)->
-        default_value(false),
-        "Perform approximate reverse transitive reduction of the marker graph.")
-
         ("MarkerGraph.peakFinder.minAreaFraction",
         value<double>(&markerGraphOptions.peakFinderMinAreaFraction)->
         default_value(0.08),
@@ -924,6 +977,178 @@ void AssemblerOptions::addConfigurableOptions()
         default_value(false),
         "Suppress output of haploid representation of the assembly (Mode 2 assembly only).")
 
+        ("Assembly.mode3.minPrimaryCoverage",
+        value<uint64_t>(&assemblyOptions.mode3Options.minPrimaryCoverage)->
+        default_value(0),
+        "Minimum primary coverage. "
+        "If minPrimaryCoverage and maxPrimaryCoverage are both 0, "
+        "they are set automatically to appropriate values using a simple heuristic."
+        "Only used with --Assembly.mode 3.")
+
+        ("Assembly.mode3.maxPrimaryCoverage",
+        value<uint64_t>(&assemblyOptions.mode3Options.maxPrimaryCoverage)->
+        default_value(0),
+        "Maximum primary coverage. "
+        "If minPrimaryCoverage and maxPrimaryCoverage are both 0, "
+        "they are set automatically to appropriate values using a simple heuristic."
+        "Only used with --Assembly.mode 3.")
+
+        ("Assembly.mode3.primaryGraph.maxLoss",
+        value<double>(&assemblyOptions.mode3Options.primaryGraphOptions.maxLoss)->
+        default_value(0.1),
+        "Use for weak edge removal in the primary graph. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.primaryGraph.crossEdgesLowCoverageThreshold",
+        value<uint64_t>(&assemblyOptions.mode3Options.primaryGraphOptions.crossEdgesLowCoverageThreshold)->
+        default_value(1),
+        "Low coverage threshold for cross edge removal in the primary graph. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.primaryGraph.crossEdgesHighCoverageThreshold",
+        value<uint64_t>(&assemblyOptions.mode3Options.primaryGraphOptions.crossEdgesHighCoverageThreshold)->
+        default_value(3),
+        "High coverage threshold for cross edge removal in the primary graph. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.detangleToleranceLow",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.detangleToleranceLow)->
+        default_value(0),
+        "Used for detangling of the assembly graph "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.detangleToleranceHigh",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.detangleToleranceHigh)->
+        default_value(2),
+        "Used for detangling of the assembly graph "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.epsilon",
+        value<double>(&assemblyOptions.mode3Options.assemblyGraphOptions.epsilon)->
+        default_value(0.1),
+        "Epsilon value for the Bayesian model used for detangling the assembly graph "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.minLogP",
+        value<double>(&assemblyOptions.mode3Options.assemblyGraphOptions.minLogP)->
+        default_value(20.),
+        "MinLogP value (in dB) for the Bayesian model used for detangling the assembly graph "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.longBubbleThreshold",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.longBubbleThreshold)->
+        default_value(5000),
+        "Long bubble threshold "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.phaseErrorThreshold",
+        value<double>(&assemblyOptions.mode3Options.assemblyGraphOptions.phaseErrorThreshold)->
+        default_value(0.1),
+        "Phase error threshold for phasing "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.bubbleErrorThreshold",
+        value<double>(&assemblyOptions.mode3Options.assemblyGraphOptions.bubbleErrorThreshold)->
+        default_value(0.03),
+        "Bubble error threshold for bubble cleanup "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.bubbleCleanupMaxOffset",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.bubbleCleanupMaxOffset)->
+        default_value(1000),
+        "Maximum bubble offset for bubble cleanup "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.chainTerminalCommonThreshold",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.chainTerminalCommonThreshold)->
+        default_value(3),
+        "Used for bubble cleanup "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.superbubbleLengthThreshold1",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.superbubbleLengthThreshold1)->
+        default_value(30000),
+        "Length threshold used for superbubble cleanup "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.superbubbleLengthThreshold2",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.superbubbleLengthThreshold2)->
+        default_value(10000),
+        "Low length threshold used for superbubble removal "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.superbubbleLengthThreshold3",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.superbubbleLengthThreshold3)->
+        default_value(30000),
+        "High length threshold used for superbubble removal "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.assemblyGraph.superbubbleLengthThreshold4",
+        value<uint64_t>(&assemblyOptions.mode3Options.assemblyGraphOptions.superbubbleLengthThreshold4)->
+        default_value(30000),
+        "Length threshold used for superbubble detangling "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.estimatedOffsetRatio",
+        value<double>(&assemblyOptions.mode3Options.localAssemblyOptions.estimatedOffsetRatio)->
+        default_value(1.1),
+        "For local assembly, the estimated offset between edgeIdA and edgeIdB gets "
+        "extended by this ratio to decide how much to extend reads that only appear in edgeIdA or edgeIdB. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.vertexSamplingRate",
+        value<double>(&assemblyOptions.mode3Options.localAssemblyOptions.vertexSamplingRate)->
+        default_value(0.8),
+        "Vertex sampling rate for local assembly, used to set minVertexCoverage. "
+        "Only used if minVertexCoverage is 0 on input to mode3::LocalAssembly constructor. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.matchScore",
+        value<int64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.matchScore)->
+        default_value(6),
+        "Match score for local assembly. (Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.mismatchScore",
+        value<int64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.mismatchScore)->
+        default_value(-1),
+        "Mismatch score for local assembly. (Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.gapScore",
+        value<int64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.gapScore)->
+        default_value(-1),
+        "Gap score for local assembly. (Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.maxSkipBases",
+        value<uint64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.maxSkipBases)->
+        default_value(500),
+        "Number of bases (not markers) that can be skipped by an alignment in local assembly. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.maxDrift",
+        value<double>(&assemblyOptions.mode3Options.localAssemblyOptions.maxDrift)->
+        default_value(0.005),
+        "The maximum tolerated length drift of each read. "
+        "Used to compute the band for banded alignments in local assembly. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.minHalfBand",
+        value<uint64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.minHalfBand)->
+        default_value(100),
+        "Minimum half band, in markers, for local assembly. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.minScoreRatio",
+        value<double>(&assemblyOptions.mode3Options.localAssemblyOptions.minScoreRatio)->
+        default_value(0.7),
+        "Score threshold for discarding alignments in for local assembly. "
+        "(Mode 3 assembly only).")
+
+        ("Assembly.mode3.localAssembly.maxMsaLength",
+        value<uint64_t>(&assemblyOptions.mode3Options.localAssemblyOptions.maxMsaLength)->
+        default_value(5000),
+        "Maximum length of a multiple sequence alignment for local assembly. "
+        "(Mode 3 assembly only).")
+
         ;
 }
 
@@ -950,6 +1175,7 @@ void ReadsOptions::write(ostream& s) const
     s << "desiredCoverage = " << desiredCoverageString << "\n";
     s << "noCache = " <<
         convertBoolToPythonString(noCache) << "\n";
+    s << "handleDuplicates = " << handleDuplicates << "\n";
     palindromicReads.write(s);
 }
 
@@ -1009,6 +1235,8 @@ void AlignOptions::write(ostream& s) const
     s << "align4.deltaY = " << align4DeltaY << "\n";
     s << "align4.minEntryCountPerCell = " << align4MinEntryCountPerCell << "\n";
     s << "align4.maxDistanceFromBoundary = " << align4MaxDistanceFromBoundary << "\n";
+    s << "align5.driftRateTolerance = " << align5DriftRateTolerance << "\n";
+    s << "align5.minBandExtend = " << align5MinBandExtend << "\n";
 }
 
 
@@ -1060,8 +1288,6 @@ void MarkerGraphOptions::write(ostream& s) const
     s << "pruneIterationCount = " << pruneIterationCount << "\n";
     s << "simplifyMaxLength = " << simplifyMaxLength << "\n";
     s << "crossEdgeCoverageThreshold = " << crossEdgeCoverageThreshold << "\n";
-    s << "reverseTransitiveReduction = " <<
-        convertBoolToPythonString(reverseTransitiveReduction) << "\n";
     s << "peakFinder.minAreaFraction = " << peakFinderMinAreaFraction << "\n";
     s << "peakFinder.areaStartIndex = " << peakFinderAreaStartIndex << "\n";
 
@@ -1105,6 +1331,7 @@ void AssemblyOptions::write(ostream& s) const
     s << "iterative.bridgeRemovalMaxDistance = " << iterativeBridgeRemovalMaxDistance << "\n";
 
     mode2Options.write(s);
+    mode3Options.write(s);
 }
 
 
@@ -1134,6 +1361,68 @@ void Mode2AssemblyOptions::write(ostream& s) const
 
 
 
+void Mode3AssemblyOptions::write(ostream& s) const
+{
+    s << "minPrimaryCoverage = " << minPrimaryCoverage << "\n";
+    s << "maxPrimaryCoverage = " << maxPrimaryCoverage << "\n";
+    primaryGraphOptions.write(s);
+    assemblyGraphOptions.write(s);
+    localAssemblyOptions.write(s);
+}
+
+
+
+void Mode3AssemblyOptions::PrimaryGraphOptions::write(ostream& s) const
+{
+    s << "mode3.primaryGraph.maxLoss = " << maxLoss << "\n";
+    s << "mode3.primaryGraph.crossEdgesLowCoverageThreshold = " << crossEdgesLowCoverageThreshold << "\n";
+    s << "mode3.primaryGraph.crossEdgesHighCoverageThreshold = " << crossEdgesHighCoverageThreshold << "\n";
+
+}
+
+
+
+void Mode3AssemblyOptions::AssemblyGraphOptions::write(ostream& s) const
+{
+    s << "mode3.assemblyGraph.detangleToleranceLow = " << detangleToleranceLow << "\n";
+    s << "mode3.assemblyGraph.detangleToleranceHigh = " << detangleToleranceHigh << "\n";
+    s << "mode3.assemblyGraph.epsilon = " << epsilon << "\n";
+    s << "mode3.assemblyGraph.minLogP = " << minLogP << "\n";
+    s << "mode3.assemblyGraph.longBubbleThreshold = " << longBubbleThreshold << "\n";
+    s << "mode3.assemblyGraph.phaseErrorThreshold = " << phaseErrorThreshold << "\n";
+    s << "mode3.assemblyGraph.bubbleErrorThreshold = " << bubbleErrorThreshold << "\n";
+    s << "mode3.assemblyGraph.bubbleCleanupMaxOffset = " << bubbleCleanupMaxOffset << "\n";
+    s << "mode3.assemblyGraph.chainTerminalCommonThreshold = " << chainTerminalCommonThreshold << "\n";
+    s << "mode3.assemblyGraph.superbubbleLengthThreshold1 = " << superbubbleLengthThreshold1 << "\n";
+    s << "mode3.assemblyGraph.superbubbleLengthThreshold2 = " << superbubbleLengthThreshold2 << "\n";
+    s << "mode3.assemblyGraph.superbubbleLengthThreshold3 = " << superbubbleLengthThreshold3 << "\n";
+    s << "mode3.assemblyGraph.superbubbleLengthThreshold4 = " << superbubbleLengthThreshold4 << "\n";
+}
+
+
+
+void Mode3AssemblyOptions::LocalAssemblyOptions::write(ostream& s) const
+{
+    s << "mode3.localAssembly.estimatedOffsetRatio = " << estimatedOffsetRatio << "\n";
+    s << "mode3.localAssembly.vertexSamplingRate = " << vertexSamplingRate << "\n";
+
+    s << "mode3.localAssembly.matchScore = " << matchScore << "\n";
+    s << "mode3.localAssembly.mismatchScore = " << mismatchScore << "\n";
+    s << "mode3.localAssembly.gapScore = " << gapScore << "\n";
+
+    s << "mode3.localAssembly.maxSkipBases = " << maxSkipBases << "\n";
+
+    s << "mode3.localAssembly.maxDrift = " << maxDrift << "\n";
+
+    s << "mode3.localAssembly.minHalfBand = " << minHalfBand << "\n";
+
+    s << "mode3.localAssembly.minScoreRatio = " << minScoreRatio << "\n";
+
+    s << "mode3.localAssembly.maxMsaLength = " << maxMsaLength << "\n";
+}
+
+
+
 void AssemblerOptions::write(ostream& s) const
 {
     readsOptions.write(s);
