@@ -76,6 +76,7 @@ namespace shasta {
     class MarkerGraphOptions;
     class MinHashOptions;
     class Mode2AssemblyOptions;
+    class Mode3AssemblyOptions;
     class PalindromicReadOptions;
     class ReadsOptions;
     class ReadGraphOptions;
@@ -127,6 +128,16 @@ public:
     bool noCache;
     string desiredCoverageString;
     uint64_t desiredCoverage;
+
+    // String to control handling of duplicate reads.
+    // Can be one of:
+    // useAllCopies
+    // useOneCopy
+    // useNone
+    // forbid
+    // See ReadFlags.hpp for the meaning of each option.
+    string handleDuplicates;
+
     PalindromicReadOptions palindromicReads;
 
     void write(ostream&) const;
@@ -195,6 +206,8 @@ public:
     uint64_t align4DeltaY;
     uint64_t align4MinEntryCountPerCell;
     uint64_t align4MaxDistanceFromBoundary;
+    double align5DriftRateTolerance;
+    uint64_t align5MinBandExtend;
     void write(ostream&) const;
 };
 
@@ -246,7 +259,6 @@ public:
     string simplifyMaxLength;
     double crossEdgeCoverageThreshold;
     vector<size_t> simplifyMaxLengthVector;
-    bool reverseTransitiveReduction;
     double peakFinderMinAreaFraction;
     uint64_t peakFinderAreaStartIndex;
 
@@ -307,6 +319,104 @@ public:
 
 
 
+// Assembly options that are specific to Mode 3 assembly.
+// See source code in the mode3 namespace
+// (source files with a mode3-) prefix for more information
+class shasta::Mode3AssemblyOptions {
+public:
+
+    uint64_t minPrimaryCoverage;
+    uint64_t maxPrimaryCoverage;
+
+    // Options used to clean up the PrimaryGraph.
+    class PrimaryGraphOptions {
+    public:
+
+        // Parameter to control removal of weak edges.
+        double maxLoss;
+
+        // Parameters to control removal of cross edges.
+        uint64_t crossEdgesLowCoverageThreshold;
+        uint64_t crossEdgesHighCoverageThreshold;
+
+        void write(ostream&) const;
+    };
+    PrimaryGraphOptions primaryGraphOptions;
+
+
+
+    class AssemblyGraphOptions {
+    public:
+
+        // Detangle tolerances.
+        uint64_t detangleToleranceLow;
+        uint64_t detangleToleranceHigh;
+
+        // Bayesian model.
+        double epsilon;
+        double minLogP;
+
+        // Other thresholds used by the mode3::AssemblyGraph
+        uint64_t longBubbleThreshold;
+        double phaseErrorThreshold;
+        double bubbleErrorThreshold;
+        uint64_t bubbleCleanupMaxOffset;
+        uint64_t chainTerminalCommonThreshold;
+        uint64_t superbubbleLengthThreshold1;
+        uint64_t superbubbleLengthThreshold2;
+        uint64_t superbubbleLengthThreshold3;
+        uint64_t superbubbleLengthThreshold4;
+
+        void write(ostream&) const;
+    };
+    AssemblyGraphOptions assemblyGraphOptions;
+
+
+
+    // Options used by class mode3::LocalAssembly
+    class LocalAssemblyOptions {
+    public:
+
+        // The estimated offset gets extended by this ratio to
+        // decide how much to extend reads that only appear in edgeIdA or edgeIdB.
+        double estimatedOffsetRatio;
+
+        // Vertex sampling rate, used to set minVertexCoverage.
+        // Only used if minVertexCoverage is 0 on input to
+        // mode3::LocalAssembly constructor.
+        double vertexSamplingRate;
+
+        // Alignment parameters.
+        int64_t matchScore;
+        int64_t mismatchScore;
+        int64_t gapScore;
+
+        // Number of bases (not markers) that can be skipped by an alignment.
+        uint64_t maxSkipBases;
+
+        // The maximum tolerated length drift of each read.
+        // Used to compute the band for banded alignments.
+        double maxDrift;
+
+        // Minimum half band, in markers.
+        uint64_t minHalfBand;
+
+        // Minimum ration of scorew to best possible score for
+        // an alignment to be used.
+        double minScoreRatio;
+
+        // The maximum length of an MSA alignment we are willing to compute.
+        uint64_t maxMsaLength;
+
+        void write(ostream&) const;
+    };
+    LocalAssemblyOptions localAssemblyOptions;
+
+    void write(ostream&) const;
+};
+
+
+
 // Options in the [Assembly] section of the configuration file.
 // Can also be entered on the command line with option names
 // beginning with "Assembly.".
@@ -343,6 +453,9 @@ public:
     // Mode 2 assembly options.
     Mode2AssemblyOptions mode2Options;
 
+    // Mode 3 assembly options.
+    Mode3AssemblyOptions mode3Options;
+
     void write(ostream&) const;
 
     // If a relative path is provided for a Bayesian consensus caller
@@ -365,9 +478,19 @@ public:
     MarkerGraphOptions markerGraphOptions;
     AssemblyOptions assemblyOptions;
 
-    // Constructor.
+    // Constructor from a command line.
+    // If the command line includes a --config option,
+    // the specified built-in configuration or configuration file
+    // is used to fill the AssemblyOptions,
+    // but values specified on the command line take precedence.
     AssemblerOptions(int argumentCount, const char** arguments);
 
+    // Constructor from a configuration file.
+    // This only fills in the configurable options specified in
+    // the given configuration file. Command line only options
+    // are left at their defaults.
+    AssemblerOptions(const string& fileName);
+
     // Add configurable options to the Boost option description object.
     void addCommandLineOnlyOptions();
     void addConfigurableOptions();
