@@ -214,11 +214,6 @@ public:
     uint64_t backwardTransferCount = 0;
     uint64_t forwardTransferCount = 0;
 
-    // Figure out if this is a bubble is caused by copy number
-    // differences in repeats of period up to maxPeriod.
-    // If this is the case, stores the shortest period for which this is true.
-    // Otherwise, stores 0 as the period.
-    void computeCopyNumberDifferencePeriod(uint64_t maxPeriod);
     uint64_t period = 0;
     string color(uint64_t branchId) const;
 
@@ -258,6 +253,7 @@ public:
         uint64_t readRepresentation,
         uint64_t k, // Marker length
         const MemoryMapped::Vector<ReadFlags>& readFlags,
+        const Reads& reads,
         const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
         MarkerGraph&,
         uint64_t pruneLength,
@@ -314,6 +310,7 @@ private:
     uint64_t readRepresentation;
     uint64_t k;
     const MemoryMapped::Vector<ReadFlags>& readFlags;
+    const Reads& reads;
     const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers;
 public:
     uint64_t getReadCount() const
@@ -447,7 +444,7 @@ private:
     AssembleParallelData assembleParallelData;
 
     // Assemble sequence for every marker graph path of a given edge.
-    void assemble(edge_descriptor);
+    void assemble(edge_descriptor, const Reads&);
 
     // Store GFA sequence in each edge.
     void storeGfaSequence();
