@@ -1609,6 +1609,51 @@ mode2.bubbleRemoval.minConcordantReadCount = 2
 
 
 
+)zzz"},
+    {"Nanopore-ncm23-May2024", R"zzz(# This assembly configuration is for nanopore reads generated using the 
+# "Experimental extremely high-accuracy, ultra-long sequencing kit"
+# from the ONT December 2023 data release:
+# https://labs.epi2me.io/gm24385_ncm23_preview/
+
+# It uses Mode 3 assembly to create a phased assembly.
+# It was only tested for a human genome at coverage 40x to 60x,
+# but it should work at lower or higher coverage,
+# within reasonable limits, because it includes some
+# provisions for coverage adaptivity.
+
+[Reads]
+representation = 0
+minReadLength = 10000
+noCache = True
+palindromicReads.deltaThreshold = 300
+
+[Kmers]
+k = 30
+probability = 0.05
+
+[MinHash]
+minHashIterationCount = 50
+minBucketSize = 0
+maxBucketSize = 0
+minFrequency = 5
+
+[Align]
+alignMethod = 5
+sameChannelReadAlignment.suppressDeltaThreshold = 30
+minAlignedMarkerCount = 1000
+minAlignedFraction = 0.9
+maxSkip = 20
+maxDrift = 10
+maxTrim = 20
+
+[ReadGraph]
+maxAlignmentCount = 20
+strandSeparationMethod = 2
+
+[Assembly]
+mode = 3
+
+
 )zzz"}
     };
 }
