@@ -33,9 +33,8 @@ LowHash0::LowHash0(
     size_t maxBucketSize,           // The maximum size for a bucket to be used.
     size_t minFrequency,            // Minimum number of minHash hits for a pair to be considered a candidate.
     size_t threadCountArgument,
-    const MemoryMapped::Vector<KmerInfo>& kmerTable,
     const Reads& reads,
-    const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
+    const MemoryMapped::VectorOfVectors<KmerId, uint64_t>& kmerIds,
     MemoryMapped::Vector<OrientedReadPair>& candidateAlignments,
     MemoryMapped::Vector< array<uint64_t, 3> >& readLowHashStatistics,
     const string& largeDataFileNamePrefix,
@@ -48,9 +47,8 @@ LowHash0::LowHash0(
     maxBucketSize(maxBucketSize),
     minFrequency(minFrequency),
     threadCount(threadCountArgument),
-    kmerTable(kmerTable),
     reads(reads),
-    markers(markers),
+    kmerIds(kmerIds),
     readLowHashStatistics(readLowHashStatistics),
     largeDataFileNamePrefix(largeDataFileNamePrefix),
     largeDataPageSize(largeDataPageSize),
@@ -71,7 +69,7 @@ LowHash0::LowHash0(
     // and each feature generates a low hash with probability hashFraction.
     // So an estimate of the total number of hashes is:
     const uint64_t totalLowHashCountEstimate =
-        uint64_t(hashFraction * double(markers.totalSize()));
+        uint64_t(hashFraction * double(kmerIds.totalSize()));
     const uint32_t leadingZeroBitCount = uint32_t(__builtin_clzl(totalLowHashCountEstimate));
     const uint32_t log2TotalLowHashCountEstimate = 64 - leadingZeroBitCount;
 
@@ -98,18 +96,11 @@ LowHash0::LowHash0(
     cout << " = " << bucketCount << " buckets. "<< endl;
 
 
-
-
-    // Create vectors containing only the k-mer ids of all markers.
-    // This is used to speed up the computation of hash functions.
-    performanceLog << timestamp << "Creating kmer ids for oriented reads." << endl;
-    createKmerIds();
-
     // Compute the threshold for a hash value to be considered low.
     hashThreshold = uint64_t(double(hashFraction) * double(std::numeric_limits<uint64_t>::max()));
 
     // The number of oriented reads, each with its own vector of markers.
-    const OrientedReadId::Int orientedReadCount = OrientedReadId::Int(markers.size());
+    const OrientedReadId::Int orientedReadCount = OrientedReadId::Int(kmerIds.size());
     const ReadId readCount = orientedReadCount / 2;
 
 
@@ -127,6 +118,9 @@ LowHash0::LowHash0(
     // Write the header of the histogram file.
     histogramCsv << "Iteration,BucketSize,BucketCount,FeatureCount\n";
 
+    // If minBucketSize and maxBucketSize are both zero,
+    // they are chosen automatically for each iteration.
+    const bool dynamicMinMaxBucketSizes = ((minBucketSize == 0) and (maxBucketSize == 0));
 
 
     // LowHash0 iteration loop.
@@ -174,7 +168,16 @@ LowHash0::LowHash0(
         setupLoadBalancing(readCount, batchSize);
         runThreads(&LowHash0::pass2ThreadFunction, threadCount);
         buckets.endPass2(false, false);
-        computeBucketHistogram();
+
+        // Compute a histogram of bucket size.
+        vector<uint64_t> bucketHistogram;
+        computeBucketHistogram(bucketHistogram);
+
+        // If dynamic adjustment of min/max bucket size was requested,
+        // do it now for this iteration, based on the current bucket size histogram.
+        if(dynamicMinMaxBucketSizes) {
+            adjustMinMaxBucketSizes(bucketHistogram);
+        }
 
         // Pass 3: inspect the buckets to find candidates.
         batchSize = 10000;
@@ -223,7 +226,7 @@ LowHash0::LowHash0(
     for(ReadId readId=0; readId<readCount; readId++) {
         const array<uint64_t, 3>& counters = readLowHashStatistics[readId];
         const uint64_t total = std::accumulate(counters.begin(), counters.end(), 0);
-        const uint64_t featureCount = markers.size(OrientedReadId(readId, 0).getValue()) - (m-1);
+        const uint64_t featureCount = kmerIds.size(OrientedReadId(readId, 0).getValue()) - (m-1);
         const double featureSampling = double(total) / double(featureCount);
         csv << readId << ",";
         csv << (reads.getFlags(readId).isPalindromic ? "Yes," : "No,");
@@ -242,13 +245,8 @@ LowHash0::LowHash0(
         }
     }
 
-
-
-    // Clean up work areas.
+    // Clean up .
     buckets.remove();
-    kmerIds.remove();
-
-
 
     // Done.
     const auto tEnd = steady_clock::now();
@@ -258,57 +256,6 @@ LowHash0::LowHash0(
 
 
 
-void LowHash0::createKmerIds()
-{
-    kmerIds.createNew(
-    	largeDataFileNamePrefix.empty() ? "" : (largeDataFileNamePrefix + "tmp-LowHash0-Markers"),
-        largeDataPageSize);
-    const ReadId orientedReadCount = ReadId(markers.size());
-    const ReadId readCount = orientedReadCount / 2;
-    kmerIds.beginPass1(orientedReadCount);
-    for(ReadId readId=0; readId!=readCount; readId++) {
-        for(Strand strand=0; strand<2; strand++) {
-            const OrientedReadId orientedReadId(readId, strand);
-            const auto markerCount = markers.size(orientedReadId.getValue());
-            kmerIds.incrementCount(orientedReadId.getValue(), markerCount);
-        }
-    }
-    kmerIds.beginPass2();
-    kmerIds.endPass2(false);
-    const size_t batchSize = 10000;
-    setupLoadBalancing(readCount, batchSize);
-    runThreads(&LowHash0::createKmerIds, threadCount);
-}
-
-
-
-// Thread function for createKmerIds.
-void LowHash0::createKmerIds(size_t threadId)
-{
-
-    // Loop over batches assigned to this thread.
-    uint64_t begin, end;
-    while(getNextBatch(begin, end)) {
-
-        // Loop over reads assigned to this batch.
-        for(ReadId readId=ReadId(begin); readId!=ReadId(end); readId++) {
-            for(Strand strand=0; strand<2; strand++) {
-                const OrientedReadId orientedReadId(readId, strand);
-                const auto orientedReadMarkers = markers[orientedReadId.getValue()];
-
-                SHASTA_ASSERT(kmerIds.size(orientedReadId.getValue()) == orientedReadMarkers.size());
-
-                auto pointer = kmerIds.begin(orientedReadId.getValue());
-                for(const CompressedMarker& marker: orientedReadMarkers) {
-                    *pointer++ = marker.kmerId;
-                }
-            }
-        }
-    }
-}
-
-
-
 // Pass1: compute the low hashes for each oriented read
 // and prepare the buckets for filling.
 void LowHash0::pass1ThreadFunction(size_t threadId)
@@ -322,7 +269,11 @@ void LowHash0::pass1ThreadFunction(size_t threadId)
 
         // Loop over oriented reads assigned to this batch.
         for(ReadId readId=ReadId(begin); readId!=ReadId(end); readId++) {
-            if(reads.getFlags(readId).isPalindromic) {
+            const ReadFlags& flags = reads.getFlags(readId);
+            if(flags.discardDueToDuplicates) {
+                continue;
+            }
+            if(flags.isPalindromic) {
                 continue;
             }
             for(Strand strand=0; strand<2; strand++) {
@@ -340,7 +291,7 @@ void LowHash0::pass1ThreadFunction(size_t threadId)
 
 
                 // Get the markers for this oriented read.
-                KmerId* kmerIdsPointer = kmerIds.begin(orientedReadId.getValue());
+                const KmerId* kmerIdsPointer = kmerIds.begin(orientedReadId.getValue());
                 const size_t featureCount = markerCount - m + 1;
 
                 // Loop over features of this oriented read.
@@ -371,7 +322,11 @@ void LowHash0::pass2ThreadFunction(size_t threadId)
 
         // Loop over oriented reads assigned to this batch.
         for(ReadId readId=ReadId(begin); readId!=ReadId(end); readId++) {
-            if(reads.getFlags(readId).isPalindromic) {
+            const ReadFlags& flags = reads.getFlags(readId);
+            if(flags.discardDueToDuplicates) {
+                continue;
+            }
+            if(flags.isPalindromic) {
                 continue;
             }
             for(Strand strand=0; strand<2; strand++) {
@@ -563,7 +518,7 @@ void LowHash0::merge(
 
 
 
-void LowHash0::computeBucketHistogram()
+void LowHash0::computeBucketHistogram(vector<uint64_t>& bucketHistogram)
 {
     threadBucketHistogram.clear();
     threadBucketHistogram.resize(threadCount);
@@ -576,7 +531,8 @@ void LowHash0::computeBucketHistogram()
     for(const vector<uint64_t>& histogram: threadBucketHistogram) {
         largestBucketSize = max(largestBucketSize, uint64_t(histogram.size()));
     }
-    vector<uint64_t> bucketHistogram(largestBucketSize, 0);
+    bucketHistogram.clear();
+    bucketHistogram.resize(largestBucketSize, 0);
     for(const vector<uint64_t>& histogram: threadBucketHistogram) {
         for(uint64_t bucketSize=0; bucketSize<histogram.size(); bucketSize++) {
             bucketHistogram[bucketSize] += histogram[bucketSize];
@@ -611,3 +567,37 @@ void LowHash0::computeBucketHistogramThreadFunction(size_t threadId)
         }
     }
 }
+
+
+
+// Adjust minBucketSize and maxBucketSize based on the current
+// bucket size histogram.
+void LowHash0::adjustMinMaxBucketSizes(const vector<uint64_t>& histogram)
+{
+    // Set minBucketSize to the lowest bucket size B0
+    // such that histogram[B0] > histogram[B0-1].
+    bool done = false;
+    for(uint64_t B0=1; B0<histogram.size(); B0++) {
+        if(histogram[B0] > histogram[B0 - 1]) {
+            minBucketSize = B0;
+            done = true;
+            break;
+        }
+    }
+    SHASTA_ASSERT(done);
+
+    // Set maxBucketSize to the largest bucket size B1 such that histogram[B1] >= histogram[B0] = histogram[minBucketSize]
+    done = false;
+    for(uint64_t B1=histogram.size()-1; B1>=minBucketSize; B1--) {
+        if(histogram[B1] >= histogram[minBucketSize]) {
+            maxBucketSize = B1;
+            done = true;
+            break;
+        }
+    }
+    SHASTA_ASSERT(done);
+
+    cout << "Automatic settings for this iteration: minBucketSize " << minBucketSize <<
+        ", maxBucketSize " << maxBucketSize << endl;
+}
+
