@@ -13,8 +13,10 @@
 namespace shasta {
 
     class Base;
-    class MarkerGraph;
     class CompressedCoverageData;
+    class CompressedMarker;
+    class MarkerGraph;
+    class Reads;
 
     extern template class MultithreadedObject<MarkerGraph>;
 }
@@ -80,6 +82,11 @@ public:
         return vertices()[vertexId];
     }
 
+    // Find out if a vertex has more than one marker on the same oriented read.
+    bool vertexHasDuplicateOrientedReadIds(
+        VertexId,
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers) const;
+
     void remove();
 
     // The global marker graph vertex corresponding to each marker.
@@ -147,7 +154,11 @@ private:
     CreateVerticesFromVertexTableData createVerticesFromVertexTableData;
 public:
 
-
+    // The disjoint sets histogram in a MemoryMapped::Vector.
+    // This is used when flagging primary marker graph edges for Mode 3 assembly.
+    // This stored pairs(coverage, frequency).
+    // Only pairs where the frequency is not zero are stored.
+    MemoryMapped::Vector< pair<uint64_t, uint64_t> > disjointSetsHistogram;
 
     // Remove marker graph vertices and update vertices and vertexTable.
     // After this is called, the only
@@ -170,6 +181,15 @@ private:
 public:
 
 
+    // Find the common KmerId for all the markers of a marker graph vertex.
+    KmerId getVertexKmerId(
+        MarkerGraphVertexId vertexId,
+        uint64_t k,
+        const Reads&,
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers
+        ) const;
+
+
 
     // The reverse complement of each vertex.
     // Indexed by VertexId.
@@ -180,7 +200,6 @@ public:
     public:
         Uint40 source;  // The source vertex (index into globalMarkerGraphVertices).
         Uint40 target;  // The target vertex (index into globalMarkerGraphVertices).
-        uint8_t coverage;   // (255 indicates 255 or more).
 
         // Flags used to mark the edge as removed from the marker graph.
         bool wasRemoved() const
@@ -220,8 +239,8 @@ public:
         // Assembly mode 2 only.
         uint8_t wasRemovedWhileSplittingSecondaryEdges : 1;
 
-        // Unused.
-        uint8_t flag6 : 1;
+        // This is set for primary edges (Mode 3 assembly only).
+        uint8_t isPrimary : 1;
 
         void clearFlags()
         {
@@ -232,15 +251,16 @@ public:
             wasAssembled = 0;
             isSecondary = 0;
             wasRemovedWhileSplittingSecondaryEdges = 0;
-            flag6 = 0;
+            isPrimary = 0;
         }
         Edge() :
             source(MarkerGraph::invalidCompressedVertexId),
-            target(MarkerGraph::invalidCompressedVertexId),
-            coverage(0)
+            target(MarkerGraph::invalidCompressedVertexId)
         {
             clearFlags();
         }
+
+        void writeFlags(ostream&) const;
     };
     MemoryMapped::Vector<Edge> edges;
     const Edge* findEdge(Uint40 source, Uint40 target) const;
@@ -264,6 +284,25 @@ public:
     EdgeId getFirstNonRemovedOutEdge(VertexId) const;
     EdgeId getFirstNonRemovedInEdge(VertexId) const;
 
+    // Find the edge that contains a given MarkerInterval.
+    EdgeId locateMarkerInterval(
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
+       const MarkerInterval&) const;
+
+    // Apply an ordinal offset in the specified direction to a given MarkerInterval
+    // and find the edge that contains the offset MarkerInterval.
+    // This assumes that we have the complete marker graph.
+    EdgeId locateMarkerIntervalWithOffset(
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
+        MarkerInterval,
+        uint32_t ordinalOffset,
+        uint64_t direction // 0=forward, 1=backward.
+        ) const;
+
+    // Find out if an edge has duplicate oriented reads
+    // in its MarkerIntervals.
+    bool edgeHasDuplicateOrientedReadIds(EdgeId) const;
+
     // The reverse complement of each edge.
     // Indexed by EdgeId.
     MemoryMapped::Vector<EdgeId> reverseComplementEdge;
@@ -327,6 +366,90 @@ public:
     // ordered by position.
     MemoryMapped::VectorOfVectors<pair<uint32_t, CompressedCoverageData>, uint64_t>
         edgeCoverageData;
+
+
+
+    // Edge sequence for each edge, for Mode 3 assembly.
+    // There are several difference compared to the consensus sequences stored above,
+    // which are not used in Mode 3 assembly:
+    // - Mode 3 assembly assumes we are not using RLE, so we don't need to store repeat counts.
+    // - Mode 3 assembly uses createMarkerGraphedgesStrict, which guarantees that
+    //   all marker interval on a marker graph edge have exactly the same sequence.
+    //   This dramatically simplifies edge sequence assembly because we can just
+    //   obtain the sequence from the first marker interval, and multiple sequence
+    //   alignment is not nedeed.
+    // - For Mode 3 assembly we assume that marker ength k is even, and
+    //   the stored edge sequence includes the last k/2 bases from the marker
+    //   of the source vertex and the first k/2 bases from the marker of
+    //   the target vertex. As a result, every edge has at least one base of sequence,
+    //   even when adjacent markers overlap. And the sequence of a path can
+    //   be obtained by just concatenating the edge sequences.
+    MemoryMapped::VectorOfVectors<Base, uint64_t> edgeSequence;
+
+    // Flag primary edges (only used for Mode 3 assembly).
+    void flagPrimaryEdges(
+        uint64_t minPrimaryCoverage,
+        uint64_t maxPrimaryCoverage,
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers,
+        uint64_t threadCount);
+private:
+    void flagPrimaryEdgesThreadFunction(uint64_t threadId);
+    bool isPrimaryEdge(
+        EdgeId,
+        uint64_t minPrimaryCoverage,
+        uint64_t maxPrimaryCoverage,
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>& markers) const;
+    class FlagPrimaryEdgesData {
+    public:
+        uint64_t minPrimaryCoverage;
+        uint64_t maxPrimaryCoverage;
+        const MemoryMapped::VectorOfVectors<CompressedMarker, uint64_t>* markersPointer;
+    };
+    FlagPrimaryEdgesData flagPrimaryEdgesData;
+
+
+#if 0
+    // PRIMARY JOURNEYS ARE NOW COMPUTED LOCALLY BY CLASS Mode3Assembler.
+    // The primary journey of an oriented read is the sequence of primary
+    // marker graph edges encountered by the oriented read.
+    // Indexed by OrientedReadId::getValue().
+    // Only used for mode 3 assembly.
+public:
+    class PrimaryJourneyEntry {
+    public:
+        array<uint32_t, 2> ordinals;
+        EdgeId edgeId;
+        bool operator<(const PrimaryJourneyEntry& that) const {
+            return ordinals[0] < that.ordinals[0];
+        }
+    };
+    MemoryMapped::VectorOfVectors<PrimaryJourneyEntry, uint64_t> primaryJourneys;
+    void createPrimaryJourneys(uint64_t orientedReadCount, uint64_t threadCount);
+    void writePrimaryJourneys();
+private:
+    void createPrimaryJourneysThreadFunction1(uint64_t threadId);
+    void createPrimaryJourneysThreadFunction2(uint64_t threadId);
+    void createPrimaryJourneysThreadFunction12(uint64_t pass);
+    void createPrimaryJourneysThreadFunction3(uint64_t threadId);
+
+public:
+
+    // Starting from a primary marker graph edge, follow the primary journeys
+    // of all oriented reads on the edge, moving forward.
+    // Find the set of MarkerGraphEdgeIds that were encountered in this way,
+    // and for each the number of times it was encountered.
+    void followPrimaryJourneysForward(
+        MarkerGraphEdgeId,
+        vector<MarkerGraphEdgeId>&,
+        vector<uint64_t>& count
+        ) const;
+    // Same, but moving backward.
+    void followPrimaryJourneysBackward(
+        MarkerGraphEdgeId,
+        vector<MarkerGraphEdgeId>&,
+        vector<uint64_t>& count
+        ) const;
+#endif
 };
 
 #endif
