@@ -12,8 +12,11 @@
 #include "deduplicate.hpp"
 #include "dset64Test.hpp"
 #include "diploidBayesianPhase.hpp"
+#include "enumeratePaths.hpp"
 #include "shastaLapack.hpp"
+#include "globalMsa.hpp"
 #include "LongBaseSequence.hpp"
+#include "longestPath.hpp"
 #include "mappedCopy.hpp"
 #include "MedianConsensusCaller.hpp"
 #include "MemoryMappedAllocator.hpp"
@@ -154,37 +157,8 @@ PYBIND11_MODULE(shasta, shastaModule)
         // K-mers.
         .def("accessKmers",
             &Assembler::accessKmers)
-        .def("writeKmers",
-            &Assembler::writeKmers,
-            arg("fileName") = "Kmers.csv")
-        .def("randomlySelectKmers",
-            &Assembler::randomlySelectKmers,
-            arg("k"),
-            arg("probability"),
-            arg("seed") = 231)
-        .def("selectKmersBasedOnFrequency",
-            &Assembler::selectKmersBasedOnFrequency,
-            arg("k"),
-            arg("markerDensity"),
-            arg("seed") = 231,
-            arg("enrichmentThreshold"),
-            arg("threadCount") = 0)
-        .def("selectKmers2",
-            &Assembler::selectKmers2,
-            arg("k"),
-            arg("markerDensity"),
-            arg("seed") = 231,
-            arg("enrichmentThreshold"),
-            arg("threadCount") = 0)
-        .def("selectKmers4",
-            &Assembler::selectKmers4,
-            arg("k"),
-            arg("markerDensity"),
-            arg("seed") = 231,
-            arg("distanceThreshold"),
-            arg("threadCount") = 0)
-
-
+        .def("accessKmerChecker",
+            &Assembler::accessKmerChecker)
 
          // Markers.
         .def("accessMarkers",
@@ -203,10 +177,6 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("readId"),
             arg("strand"),
             arg("fileName"))
-        .def("getMarkers",
-            &Assembler::getMarkers)
-        .def("writeMarkerFrequency",
-            &Assembler::writeMarkerFrequency)
         .def("computeSortedMarkers",
             &Assembler::computeSortedMarkers,
             arg("threadCount") = 0)
@@ -226,16 +196,6 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("maxBucketSize"),
             arg("minFrequency"),
             arg("threadCount") = 0)
-        .def("findAlignmentCandidatesLowHash1",
-            &Assembler::findAlignmentCandidatesLowHash1,
-            arg("m"),
-            arg("hashFraction"),
-            arg("minHashIterationCount"),
-            arg("log2MinHashBucketCount") = 0,
-            arg("minBucketSize"),
-            arg("maxBucketSize"),
-            arg("minFrequency"),
-            arg("threadCount") = 0)
         .def("accessAlignmentCandidates",
             &Assembler::accessAlignmentCandidates)
         .def("accessAlignmentCandidateTable",
@@ -263,8 +223,6 @@ PYBIND11_MODULE(shasta, shastaModule)
              &Assembler::writeAlignmentCandidates,
              arg("useReadName") = false,
              arg("verbose") = false)
-        .def("writeAlignmentDetails",
-             &Assembler::writeAlignmentDetails)
         .def("writeLocalAlignmentCandidateReads",
              &Assembler::writeLocalAlignmentCandidateReads,
              arg("readId"),
@@ -305,7 +263,7 @@ PYBIND11_MODULE(shasta, shastaModule)
                 (ReadId, Strand, ReadId, Strand,
                     uint64_t, uint64_t, uint64_t, uint64_t,
                     uint64_t, double, uint64_t, uint64_t, uint64_t, uint64_t,
-                    int64_t, int64_t, int64_t) const
+                    int64_t, int64_t, int64_t)
             )
             &Assembler::alignOrientedReads4,
             arg("readId0"),
@@ -355,6 +313,10 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("strand0"),
             arg("readId1"),
             arg("strand1"))
+        .def("computeMarkerKmerIds",
+            &Assembler::computeMarkerKmerIds)
+        .def("cleanupMarkerKmerIds",
+            &Assembler::cleanupMarkerKmerIds)
 
 
 
@@ -437,6 +399,8 @@ PYBIND11_MODULE(shasta, shastaModule)
         .def("accessMarkerGraphVertices",
              &Assembler::accessMarkerGraphVertices,
              arg("readWriteAccess") = false)
+        .def("accessDisjointSetsHistogram",
+             &Assembler::accessDisjointSetsHistogram)
         .def("getGlobalMarkerGraphVertex",
             (
                 MarkerGraph::VertexId (Assembler::*)
@@ -481,8 +445,6 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("pattern2CreateNewVertices"))
         .def("getMarkerGraphMinCoverageUsed",
             &Assembler::getMarkerGraphMinCoverageUsed)
-        .def("vertexCoverageStatisticsByKmerId",
-            &Assembler::vertexCoverageStatisticsByKmerId)
 
         // Edges of the global marker graph.
         .def("createMarkerGraphEdges",
@@ -520,11 +482,6 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("highCoverageThreshold"),
             arg("maxDistance"),
             arg("edgeMarkerSkipThreshold"))
-        .def("reverseTransitiveReduction",
-            &Assembler::reverseTransitiveReduction,
-            arg("lowCoverageThreshold"),
-            arg("highCoverageThreshold"),
-            arg("maxDistance"))
         .def("pruneMarkerGraphStrongSubgraph",
             &Assembler::pruneMarkerGraphStrongSubgraph,
             arg("iterationCount"))
@@ -621,7 +578,7 @@ PYBIND11_MODULE(shasta, shastaModule)
         .def("assembleAssemblyGraphEdge",
             (
                 AssembledSegment (Assembler::*)
-                (AssemblyGraph::EdgeId, bool)
+                (mode0::AssemblyGraph::EdgeId, bool)
             )
             &Assembler::assembleAssemblyGraphEdge,
             arg("edgeId"),
@@ -638,8 +595,6 @@ PYBIND11_MODULE(shasta, shastaModule)
             arg("diagonalReadCountMin"),
             arg("offDiagonalReadCountMax"),
             arg("offDiagonalRatio"))
-        .def("alignPseudoPaths",
-            &Assembler::alignPseudoPaths)
         .def("removeAssemblyGraph",
             &Assembler::removeAssemblyGraph)
 
@@ -655,18 +610,11 @@ PYBIND11_MODULE(shasta, shastaModule)
 
         // Assembly mode 3.
         .def("mode3Assembly",
-            &Assembler::mode3Assembly,
-            arg("threadCount") = 0)
-        .def("accessMode3AssemblyGraph",
-            &Assembler::accessMode3AssemblyGraph)
-        .def("analyzeMode3Subgraph",
-            &Assembler::analyzeMode3Subgraph)
-        .def("createMode3PathGraph",
-            &Assembler::createMode3PathGraph)
-        .def("createMode3Detangler",
-            &Assembler::createMode3Detangler)
-
-
+            &Assembler::mode3Assembly)
+        .def("mode3AssembleComponent",
+            &Assembler::mode3AssembleComponent)
+        .def("flagPrimaryMarkerGraphEdges",
+            &Assembler::flagPrimaryMarkerGraphEdges)
 
         // Consensus caller.
         .def("setupConsensusCaller",
@@ -708,6 +656,18 @@ PYBIND11_MODULE(shasta, shastaModule)
 
 
 
+    // Expose portions of class AssemblerOptions to Python.
+    class_<AssemblerOptions>(shastaModule, "AssemblerOptions")
+        .def(pybind11::init<const string&>())
+        .def_readonly("assemblyOptions", &AssemblerOptions::assemblyOptions)
+        ;
+    class_<AssemblyOptions>(shastaModule, "AssemblyOptions")
+        .def_readonly("mode3Options", &AssemblyOptions::mode3Options)
+        ;
+    class_<Mode3AssemblyOptions>(shastaModule, "Mode3AssemblyOptions");
+
+
+
     // Constants.
     shastaModule.attr("invalidGlobalMarkerGraphVertexId") = MarkerGraph::invalidVertexId;
     shastaModule.attr("invalidCompressedGlobalMarkerGraphVertexId") =
@@ -785,6 +745,15 @@ PYBIND11_MODULE(shasta, shastaModule)
     shastaModule.def("testSubsetGraph",
         testSubsetGraph
         );
+    shastaModule.def("testLongestPath",
+        testLongestPath
+        );
+    shastaModule.def("testEnumeratePaths",
+        testEnumeratePaths
+        );
+    shastaModule.def("globalMsaPython",
+        globalMsaPython
+        );
 }
 
 #endif
