@@ -1,6 +1,7 @@
 // Shasta.
 #include "ReadLoader.hpp"
 #include "computeRunLengthRepresentation.hpp"
+#include "filesystem.hpp"
 #include "performanceLog.hpp"
 #include "splitRange.hpp"
 using namespace shasta;
@@ -11,6 +12,8 @@ using namespace shasta;
 #include <filesystem>
 #include "tuple.hpp"
 
+#include "MultithreadedObject.tpp"
+template class MultithreadedObject<ReadLoader>;
 
 // Load reads from a fastq or fasta file.
 ReadLoader::ReadLoader(
@@ -22,7 +25,7 @@ ReadLoader::ReadLoader(
     const string& dataNamePrefix,
     size_t pageSize,
     Reads& reads):
-    
+
     MultithreadedObject(*this),
     fileName(fileName),
     representation(representation),
@@ -511,14 +514,12 @@ void ReadLoader::allocateBufferAndReadFile()
 
     // If there was failure and we are using noCache, try turning it off.
     // If successful, all done.
-#ifdef __linux__
     if(noCache) {
         cout << "Turning off --Reads.noCache for " << fileName << endl;
         if(readFile(false)) {
             return;
         }
     }
-#endif
 
     // If getting here, nothing worked.
     throw runtime_error("Error reading " + fileName);
@@ -554,11 +555,9 @@ bool ReadLoader::readFile(bool useODirect)
 
     // Set up flags to open the file.
     int flags = O_RDONLY;
-#ifdef __linux__
     if(useODirect) {
         flags |= O_DIRECT;
     }
-#endif
 
     // Open the input file.
     const int fileDescriptor = ::open(fileName.c_str(), flags);
