@@ -176,6 +176,7 @@ function zoomSvg(factor)
     y = yCenter - 0.5 * height;
 
     svg.setAttribute('viewBox', `${x} ${y} ${width} ${height}`);
+    svg.setAttribute('font-size', svg.getAttribute('font-size') / factor);
 
     return false;
 }
@@ -185,3 +186,10 @@ function zoomSvg(factor)
 }
 
 
+
+
+void shasta::writeInformationIcon(ostream& html, const string& message)
+{
+    html << "<span style='color:Blue;font-weight:bold' title=\"" <<
+            message << "\">&#9432;</span>";
+}
