@@ -1,23 +1,12 @@
 #include "platformDependent.hpp"
-#ifdef __linux__
 #include <stdlib.h>
 #include "fstream.hpp"
-#endif
 
 // Return the path to a usable temporary directory, including the final "/".
 std::string shasta::tmpDirectory()
 {
-#ifdef __linux__
-
-    // For Linux, use /dev/shm so the temporary files never go to disk.
+    // Use /dev/shm, so the temporary files never go to disk.
     return "/dev/shm/";
-    
-#else
-
-    // For macOS, use environment variable TMPDIR.
-    return string(::getenv("TMPDIR")) + "/";
-    
-#endif
 }
 
 
@@ -25,21 +14,14 @@ std::string shasta::tmpDirectory()
 // Return the name of a timeout command or equivalent.
 std::string shasta::timeoutCommand()
 {
-#ifdef __linux__
     return "timeout";
-#else
-
-    // For macOS, return gtimeout.
-    // Requires brew install coreutils.
-    return "gtimeout";
-    
-#endif
-    
 }
 
+
+
 uint64_t shasta::getPeakMemoryUsage() {
     uint64_t peakMemoryUsage = 0ULL;
-#ifdef __linux__
+
     ifstream procStats("/proc/self/status");
     if (procStats) {
         string line;
@@ -58,6 +40,19 @@ uint64_t shasta::getPeakMemoryUsage() {
             break;
         }
     }
-#endif
+
     return peakMemoryUsage;
 }
+
+
+
+// Get total physical memory available, in bytes.
+uint64_t shasta::getTotalPhysicalMemory()
+{
+    ifstream meminfo("/proc/meminfo");
+    string s;
+    uint64_t memoryKb;
+    meminfo >> s >> memoryKb;
+    return 1024 * memoryKb;
+}
+
