@@ -1,4 +1,3 @@
-// Main program for the Shasta static executable.
 // The static executable provides
 // basic functionality and reduced performance.
 // For full functionality use the shared library built
@@ -220,6 +219,18 @@ void shasta::main::assemble(
             "and is now required to run an assembly.");
     }
 
+    // Check --Kmers.k.
+    // Using Kmer=ShortBaseSequence16 limits it to 16 bases.
+    // But alignment methods adds 100 to KmerIds to deal
+    // with the Seqan gap value 45, so this means
+    // that we cannot use k=16.
+    // Therefore the maximum allowed value is 15.
+    // We also reject values that are grossly too low.
+    if(assemblerOptions.kmersOptions.k > 31 or assemblerOptions.kmersOptions.k < 6) {
+        throw runtime_error("Invalid value specified for --Kmers.k. "
+            "Must be between 6 and 31");
+    }
+
     // Check that we have at least one input file.
     if(assemblerOptions.commandLineOnlyOptions.inputFileNames.empty()) {
         cout << assemblerOptions.allOptionsDescription << endl;
@@ -228,17 +239,19 @@ void shasta::main::assemble(
     }
 
     // Check assemblerOptions.minHashOptions.version.
-    if( assemblerOptions.minHashOptions.version!=0 and
-        assemblerOptions.minHashOptions.version!=1) {
+    if( assemblerOptions.minHashOptions.version!=0) {
         throw runtime_error("Invalid value " +
             to_string(assemblerOptions.minHashOptions.version) +
-            " specified for --MinHash.version. Must be 0 or 1.");
+            " specified for --MinHash.version. Must be 0.");
     }
 
     // Check assemblerOptions.minHashOptions minimum/maximum bucket size.
-    if( assemblerOptions.minHashOptions.maxBucketSize <=
-        assemblerOptions.minHashOptions.minBucketSize) {
-        throw runtime_error("MinHash maximum bucket size must be greater than minimum bucket size. "
+    const bool dynamicMinHashBucketRange =
+        (assemblerOptions.minHashOptions.minBucketSize == 0) and
+        (assemblerOptions.minHashOptions.maxBucketSize == 0);
+    if((not dynamicMinHashBucketRange) and (assemblerOptions.minHashOptions.maxBucketSize <=
+        assemblerOptions.minHashOptions.minBucketSize)) {
+        throw runtime_error("Invalid MinHash min/max bucket sizes specified. "
             "The following values were specified:"
             " minimum bucket size " +
             to_string(assemblerOptions.minHashOptions.minBucketSize) +
@@ -258,9 +271,9 @@ void shasta::main::assemble(
 
     if( assemblerOptions.alignOptions.alignMethod <  0 or
         assemblerOptions.alignOptions.alignMethod == 2 or
-        assemblerOptions.alignOptions.alignMethod >  4) {
+        assemblerOptions.alignOptions.alignMethod >  5) {
         throw runtime_error("Align method " + to_string(assemblerOptions.alignOptions.alignMethod) +
-            " is not valid. Valid options are 0, 1, 3, and 4.");
+            " is not valid. Valid options are 0, 1, 3, 4, and 5.");
     }
 
     if(assemblerOptions.readGraphOptions.creationMethod != 0 and
@@ -284,6 +297,10 @@ void shasta::main::assemble(
         assemblerOptions.readGraphOptions.strandSeparationMethod != 2) {
         throw runtime_error("--Assembly.mode 2 requires --ReadGraph.strandSeparationMethod 2.");
     }
+    if(assemblerOptions.assemblyOptions.mode == 3 and
+        assemblerOptions.readGraphOptions.strandSeparationMethod != 2) {
+        throw runtime_error("--Assembly.mode 3 requires --ReadGraph.strandSeparationMethod 2.");
+    }
 
     // Find absolute paths of the input files.
     // We will use them below after changing directory to the output directory.
@@ -370,11 +387,12 @@ void shasta::main::assemble(
         assemblerOptions.commandLineOnlyOptions.memoryMode != "filesystem") {
         cout << "This run uses options \"--memoryBacking " << assemblerOptions.commandLineOnlyOptions.memoryBacking <<
             " --memoryMode " << assemblerOptions.commandLineOnlyOptions.memoryMode << "\".\n"
-            "This could result in performance degradation.\n"
-            "For full performance, use \"--memoryBacking 2M --memoryMode filesystem\"\n"
+            "This could result in longer run time.\n"
+            "For faster assembly, use \"--memoryBacking 2M --memoryMode filesystem\"\n"
             "(root privilege via sudo required).\n"
             "Therefore the results of this run should not be used\n"
-            "for benchmarking purposes." << endl;
+            "for the purpose of benchmarking assembly time.\n"
+            "However the memory options don't affect assembly results in any way." << endl;
     }
 
     // Create the Assembler.
@@ -391,11 +409,12 @@ void shasta::main::assemble(
         assemblerOptions.commandLineOnlyOptions.memoryMode != "filesystem") {
         cout << "This run used options \"--memoryBacking " << assemblerOptions.commandLineOnlyOptions.memoryBacking <<
             " --memoryMode " << assemblerOptions.commandLineOnlyOptions.memoryMode << "\".\n"
-            "This could have resulted in performance degradation.\n"
-            "For full performance, use \"--memoryBacking 2M --memoryMode filesystem\"\n"
+            "This could result in longer run time.\n"
+            "For faster assembly, use \"--memoryBacking 2M --memoryMode filesystem\"\n"
             "(root privilege via sudo required).\n"
             "Therefore the results of this run should not be used\n"
-            "for benchmarking purposes." << endl;
+            "for the purpose of benchmarking assembly time.\n"
+            "However the memory options don't affect assembly results in any way." << endl;
     }
 
     // Write out the build id again.
@@ -530,8 +549,10 @@ void shasta::main::assemble(
     cout << "This assembly will use " << threadCount << " threads." << endl;
 
     // Set up the consensus caller.
-    cout << "Setting up consensus caller " <<
-        assemblerOptions.assemblyOptions.consensusCaller << endl;
+    if(assembler.getReads().representation == 1) {
+        cout << "Setting up consensus caller " <<
+            assemblerOptions.assemblyOptions.consensusCaller << endl;
+    }
     assembler.setupConsensusCaller(assemblerOptions.assemblyOptions.consensusCaller);
 
 
@@ -588,71 +609,25 @@ void shasta::main::assemble(
     performanceLog << timestamp << "Done loading reads from " << inputFileNames.size() << " files." << endl;
     performanceLog << "Read loading took " << seconds(t1-t0) << "s." << endl;
 
+    // Find duplicate reads and handle them according to the setting
+    // of --Reads.handleDuplicates.
+    assembler.findDuplicateReads(assemblerOptions.readsOptions.handleDuplicates);
 
-
-    // Select the k-mers that will be used as markers.
-    switch(assemblerOptions.kmersOptions.generationMethod) {
-    case 0:
-        assembler.randomlySelectKmers(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231);
-        break;
-
-    case 1:
-        // Randomly select the k-mers to be used as markers, but
-        // excluding those that are globally overenriched in the input reads,
-        // as measured by total frequency in all reads.
-        assembler.selectKmersBasedOnFrequency(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231,
-            assemblerOptions.kmersOptions.enrichmentThreshold, threadCount);
-        break;
-
-    case 2:
-        // Randomly select the k-mers to be used as markers, but
-        // excluding those that are overenriched even in a single oriented read.
-        assembler.selectKmers2(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231,
-            assemblerOptions.kmersOptions.enrichmentThreshold, threadCount);
-        break;
-
-    case 3:
-        // Read the k-mers to be used as markers from a file.
-        if(assemblerOptions.kmersOptions.file.empty() or
-            assemblerOptions.kmersOptions.file[0] != '/') {
-            throw runtime_error("Option --Kmers.file must specify an absolute path. "
-                "A relative path is not accepted.");
-        }
-        assembler.readKmersFromFile(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.file);
-        break;
-
-    case 4:
-        // Randomly select the k-mers to be used as markers, but
-        // excluding those that appear in two copies close to each other
-        // even in a single oriented read.
-        assembler.selectKmers4(
-            assemblerOptions.kmersOptions.k,
-            assemblerOptions.kmersOptions.probability, 231,
-            assemblerOptions.kmersOptions.distanceThreshold, threadCount);
-        break;
-
-    default:
-        throw runtime_error("Invalid --Kmers generationMethod. "
-            "Specify a value between 0 and 4, inclusive.");
-    }
-
-
+    // Initialize the KmerChecker, which has the information needed
+    // to decide if a k-mer is a marker.
+    assembler.createKmerChecker(assemblerOptions.kmersOptions, threadCount);
 
     // Find the markers in the reads.
     assembler.findMarkers(0);
 
-    if(!assemblerOptions.readsOptions.palindromicReads.skipFlagging) {
+    // Gather marker KmerIds for all markers.
+    // They are used by LowHash and alignment computation.
+    // These will be kept until we are done computing alignments.
+    assembler.computeMarkerKmerIds(threadCount);
 
-        // Flag palindromic reads.
-        // These will be excluded from further processing.
+    // Flag palindromic reads.
+    // These will be excluded from further processing.
+    if(!assemblerOptions.readsOptions.palindromicReads.skipFlagging) {
         assembler.flagPalindromicReads(
             assemblerOptions.readsOptions.palindromicReads.maxSkip,
             assemblerOptions.readsOptions.palindromicReads.maxDrift,
@@ -663,12 +638,11 @@ void shasta::main::assemble(
             threadCount);
     }
 
-
-
     // Find alignment candidates.
     if(assemblerOptions.minHashOptions.allPairs) {
         assembler.markAlignmentCandidatesAllPairs();
-    } else if(assemblerOptions.minHashOptions.version == 0) {
+    } else {
+        SHASTA_ASSERT(assemblerOptions.minHashOptions.version == 0); // Already checked for that.
         assembler.findAlignmentCandidatesLowHash0(
             assemblerOptions.minHashOptions.m,
             assemblerOptions.minHashOptions.hashFraction,
@@ -679,17 +653,6 @@ void shasta::main::assemble(
             assemblerOptions.minHashOptions.maxBucketSize,
             assemblerOptions.minHashOptions.minFrequency,
             threadCount);
-    } else {
-        SHASTA_ASSERT(assemblerOptions.minHashOptions.version == 1);    // Already checked for that.
-        assembler.findAlignmentCandidatesLowHash1(
-            assemblerOptions.minHashOptions.m,
-            assemblerOptions.minHashOptions.hashFraction,
-            assemblerOptions.minHashOptions.minHashIterationCount,
-            0,
-            assemblerOptions.minHashOptions.minBucketSize,
-            assemblerOptions.minHashOptions.maxBucketSize,
-            assemblerOptions.minHashOptions.minFrequency,
-            threadCount);
     }
 
 
@@ -711,6 +674,9 @@ void shasta::main::assemble(
         assemblerOptions.alignOptions,
         threadCount);
 
+    // Marker KmerIds are freed here.
+    // They can always be recomputed from the reads when needed.
+    assembler.cleanupMarkerKmerIds();
 
 
     // Create the read graph.
@@ -943,14 +909,6 @@ void shasta::main::mode0Assembly(
         assemblerOptions.markerGraphOptions.highCoverageThreshold,
         assemblerOptions.markerGraphOptions.maxDistance,
         assemblerOptions.markerGraphOptions.edgeMarkerSkipThreshold);
-    if(assemblerOptions.markerGraphOptions.reverseTransitiveReduction) {
-        assembler.reverseTransitiveReduction(
-            assemblerOptions.markerGraphOptions.lowCoverageThreshold,
-            assemblerOptions.markerGraphOptions.highCoverageThreshold,
-            assemblerOptions.markerGraphOptions.maxDistance);
-    }
-
-
 
     // Prune the marker graph.
     assembler.pruneMarkerGraphStrongSubgraph(
@@ -1118,52 +1076,54 @@ void shasta::main::mode3Assembly(
     const AssemblerOptions& assemblerOptions,
     uint32_t threadCount)
 {
+    // Mode 3 assembly requires reads in raw representation (not RLE).
+    SHASTA_ASSERT(assemblerOptions.readsOptions.representation == 0);
+
+    // The marker length must be even.
+    SHASTA_ASSERT((assembler.assemblerInfo->k %2) == 0);
+
     // Create marker graph vertices.
+    // To create a complete marker graph, generate all vertices
+    // regardless of coverage, and allow duplicate markers on vertices.
     assembler.createMarkerGraphVertices(
-        assemblerOptions.markerGraphOptions.minCoverage,
-        assemblerOptions.markerGraphOptions.maxCoverage,
-        assemblerOptions.markerGraphOptions.minCoveragePerStrand,
-        assemblerOptions.markerGraphOptions.allowDuplicateMarkers,
-        assemblerOptions.markerGraphOptions.peakFinderMinAreaFraction,
-        assemblerOptions.markerGraphOptions.peakFinderAreaStartIndex,
+        1,                                              // minVertexCoverage
+        std::numeric_limits<uint64_t>::max(),           // maxVertexCoverage
+        0,                                              // minVertexCoveragePerStrand
+        true,                                           // allowDuplicateMarkers
+        std::numeric_limits<double>::signaling_NaN(),   // For peak finder, unused because minVertexCoverage is not 0.
+        invalid<uint64_t>,                              // For peak finder, unused because minVertexCoverage is not 0.
         threadCount);
     assembler.findMarkerGraphReverseComplementVertices(threadCount);
 
     // Create marker graph edges.
-    // For assembly mode 1 we use createMarkerGraphEdgesStrict
-    // with minimum edge coverage (total and per strand).
+    // Use createMarkerGraphEdgesStrict so all oriented reads on an edge
+    // have exactly the same sequence.
+    // To create a complete marker graph, generate all edges
+    // regardless of coverage.
     assembler.createMarkerGraphEdgesStrict(
-        assemblerOptions.markerGraphOptions.minEdgeCoverage,
-        assemblerOptions.markerGraphOptions.minEdgeCoveragePerStrand, threadCount);
+        0,                      // minEdgeCoverage
+        0,                      // minEdgeCoveragePerStrand
+        threadCount);
     assembler.findMarkerGraphReverseComplementEdges(threadCount);
 
     // Coverage histograms for vertices and edges of the marker graph.
     assembler.computeMarkerGraphCoverageHistogram();
 
-    // In mode 3 assembly, we don't add secondary edges.
+    // Assemble sequence for marker graph edges.
+    // This assembles MarkerGraph::edgeSequence which is
+    // different from what happens in other assembly modes.
+    // See the comments before MarkerGraph::edgeSequence
+    // for more information.
+    assembler.assembleMarkerGraphEdgesMode3();
 
-    // Coverage histograms for vertices and edges of the marker graph.
-    assembler.computeMarkerGraphCoverageHistogram();
-
-    // Compute optimal repeat counts for each vertex of the marker graph.
-    if(assemblerOptions.readsOptions.representation == 1) {
-        assembler.assembleMarkerGraphVertices(threadCount);
-    }
-
-    // Compute consensus sequence for all marker graph edges.
-    assembler.assembleMarkerGraphEdges(
-        threadCount,
-        assemblerOptions.assemblyOptions.markerGraphEdgeLengthThresholdForConsensus,
-        assemblerOptions.assemblyOptions.storeCoverageData or
-        assemblerOptions.assemblyOptions.storeCoverageDataCsvLengthThreshold>0,
-        true
-        );
-
-    // Run mode 3 assembly.
-    assembler.mode3Assembly(
+    // Flag primary marker graph edges.
+    assembler.flagPrimaryMarkerGraphEdges(
+        assemblerOptions.assemblyOptions.mode3Options.minPrimaryCoverage,
+        assemblerOptions.assemblyOptions.mode3Options.maxPrimaryCoverage,
         threadCount);
 
-
+    // Run Mode 3 assembly.
+    assembler.mode3Assembly(threadCount, assemblerOptions.assemblyOptions.mode3Options, false);
 }
 
 
@@ -1321,8 +1281,10 @@ void shasta::main::explore(
     Assembler assembler("Data/", false, 1, 0);
 
     // Set up the consensus caller.
-    cout << "Setting up consensus caller " <<
-        assemblerOptions.assemblyOptions.consensusCaller << endl;
+    if(assembler.getReads().representation == 1) {
+        cout << "Setting up consensus caller " <<
+            assemblerOptions.assemblyOptions.consensusCaller << endl;
+    }
     assembler.setupConsensusCaller(assemblerOptions.assemblyOptions.consensusCaller);
 
     // Access all available binary data.
